@@ -146,11 +146,16 @@ pub enum Action {
     Edit,
     /// Reset temporary recipe override to its default value
     Reset,
+    /// Open content in the configured external pager
+    View,
     /// Browse request history
     History,
     /// Start a search/filter operation
     #[display("Search/Filter")]
     Search,
+    /// Enter a command to export data
+    #[display("Export")]
+    Export,
     /// Force a collection reload (typically it's automatic)
     #[display("Reload Collection")]
     ReloadCollection,
