@@ -10,11 +10,13 @@
 //! the compile chain a bit further.
 
 mod input;
+mod mime;
 mod theme;
 
 pub use input::{Action, InputBinding, KeyCombination};
 pub use theme::Theme;
 
+use crate::mime::MimeMap;
 use anyhow::Context;
 use indexmap::IndexMap;
 use serde::{Deserialize, Serialize};
@@ -39,11 +41,20 @@ const FILE: &str = "config.yml";
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(default, deny_unknown_fields)]
 pub struct Config {
+    /// Configuration for in-app query and export commands
+    pub commands: CommandsConfig,
     /// Command to use for in-app editing. If provided, overrides
-    /// `VISUAL`/`EDITOR` environment variables
+    /// `VISUAL`/`EDITOR` environment variables. This only supports a single
+    /// command, *not* a content type map. This is because there isn't much
+    /// value in it, and plumbing the content type around to support it is
+    /// annoying.
     pub editor: Option<String>,
-    /// Command to use to browse response bodies
-    pub viewer: Option<String>,
+    /// Command to use to browse response bodies. If provided, overrides
+    /// `PAGER` environment variable.  This could be a single command, or a map
+    /// of {content_type: command} to use different commands based on response
+    /// type. Aliased for backward compatibility with the old name.
+    #[serde(alias = "viewer", default)]
+    pub pager: MimeMap<String>,
     #[serde(flatten)]
     pub http: HttpEngineConfig,
     /// Should templates be rendered inline in the UI, or should we show the
@@ -112,9 +123,10 @@ impl Config {
 impl Default for Config {
     fn default() -> Self {
         Self {
-            editor: None,
-            viewer: None,
-            http: HttpEngineConfig::default(),
+            commands: CommandsConfig::default(),
+            editor: Default::default(),
+            pager: Default::default(),
+            http: Default::default(),
             preview_templates: true,
             input_bindings: Default::default(),
             theme: Default::default(),
@@ -123,6 +135,36 @@ impl Default for Config {
     }
 }
 
+/// Configuration for in-app query and export commands
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(default, deny_unknown_fields)]
+pub struct CommandsConfig {
+    /// Wrapping shell to parse and execute commands
+    /// If empty, commands will be parsed with shell-words and run natievly
+    pub shell: Vec<String>,
+    /// Default query command for responses
+    #[serde(default)]
+    pub default_query: MimeMap<String>,
+}
+
+impl Default for CommandsConfig {
+    fn default() -> Self {
+        // We use the defaults from docker, because it's well tested and
+        // reasonably intuitive
+        // https://docs.docker.com/reference/dockerfile/#shell
+        let default_shell: &[&str] = if cfg!(windows) {
+            &["cmd", "/S", "/C"]
+        } else {
+            &["/bin/sh", "-c"]
+        };
+
+        Self {
+            shell: default_shell.iter().map(|s| s.to_string()).collect(),
+            default_query: MimeMap::default(),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
