@@ -19,7 +19,7 @@ use serde::{
     ser::Error as _,
     Deserialize, Deserializer, Serialize, Serializer,
 };
-use std::{fmt::Display, hash::Hash, str::FromStr};
+use std::{hash::Hash, str::FromStr};
 
 /// A type that has an `id` field. This is ripe for a derive macro, maybe a fun
 /// project some day?
@@ -106,17 +106,6 @@ where
     Ok(map)
 }
 
-/// Deserialize a value using its `FromStr` implementation
-pub fn deserialize_from_str<'de, D, T>(deserializer: D) -> Result<T, D::Error>
-where
-    D: Deserializer<'de>,
-    T: FromStr,
-    T::Err: Display,
-{
-    let s = String::deserialize(deserializer)?;
-    s.parse().map_err(D::Error::custom)
-}
-
 /// Deserialize a profile mapping. This also enforces that only one profile is
 /// marked as default
 pub fn deserialize_profiles<'de, D>(
@@ -237,6 +226,25 @@ pub mod serde_query_parameters {
     }
 }
 
+/// Deserialize a header map, lowercasing all header names. Headers are
+/// case-insensitive (and must be lowercase in HTTP/2+), so forcing the case
+/// makes lookups on the map easier.
+pub fn deserialize_headers<'de, D>(
+    deserializer: D,
+) -> Result<IndexMap<String, Template>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    // This involves an extra allocation, but it makes the logic a lot easier.
+    // These maps should be small anyway
+    let headers: IndexMap<String, Template> =
+        IndexMap::deserialize(deserializer)?;
+    Ok(headers
+        .into_iter()
+        .map(|(k, v)| (k.to_ascii_lowercase(), v))
+        .collect())
+}
+
 impl RecipeBody {
     // Constants for serialize/deserialization. Typically these are generated
     // by macros, but we need custom implementation
