@@ -5,17 +5,17 @@ use crate::{
         cereal,
         recipe_tree::{RecipeNode, RecipeTree},
     },
-    http::{content_type::ContentType, query::Query},
+    http::{content_type::ContentType, query::Query, HttpMethod},
     template::{Identifier, Template},
     util::{parse_yaml, ResultTraced},
 };
-use anyhow::{anyhow, Context};
+use anyhow::Context;
 use derive_more::{Deref, Display, From, FromStr};
 use indexmap::IndexMap;
-use itertools::Itertools;
+use mime::Mime;
+use reqwest::header;
 use serde::{Deserialize, Serialize};
 use std::{fs::File, path::PathBuf, time::Duration};
-use strum::{EnumIter, IntoEnumIterator};
 use tracing::info;
 
 /// A collection of profiles, requests, etc. This is the primary Slumber unit
@@ -55,7 +55,7 @@ impl Collection {
         };
 
         load()
-            .context(format!("Error loading data from {path:?}"))
+            .context(format!("Error loading collection from {path:?}"))
             .traced()
     }
 }
@@ -113,6 +113,8 @@ impl crate::test_util::Factory for Profile {
     Serialize,
     Deserialize,
 )]
+#[deref(forward)]
+#[serde(transparent)]
 pub struct ProfileId(String);
 
 #[cfg(any(test, feature = "test"))]
@@ -164,11 +166,48 @@ impl crate::test_util::Factory for Folder {
     }
 }
 
+/// A definition of how to make a request. This is *not* called `Request` in
+/// order to distinguish it from a single instance of an HTTP request. And it's
+/// not called `RequestTemplate` because the word "template" has a specific
+/// meaning related to string interpolation.
+#[derive(Debug, Serialize, Deserialize)]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(deny_unknown_fields)]
+pub struct Recipe {
+    #[serde(skip)] // This will be auto-populated from the map key
+    pub id: RecipeId,
+    pub name: Option<String>,
+    /// *Not* a template string because the usefulness doesn't justify the
+    /// complexity. This gives the user an immediate error if the method is
+    /// wrong which is helpful.
+    pub method: HttpMethod,
+    pub url: Template,
+    pub body: Option<RecipeBody>,
+    pub authentication: Option<Authentication>,
+    #[serde(default, with = "cereal::serde_query_parameters")]
+    pub query: Vec<(String, Template)>,
+    #[serde(default, deserialize_with = "cereal::deserialize_headers")]
+    pub headers: IndexMap<String, Template>,
+}
+
 impl Recipe {
     /// Get a presentable name for this recipe
     pub fn name(&self) -> &str {
         self.name.as_deref().unwrap_or(&self.id)
     }
+
+    /// Guess the value that the `Content-Type` header will have for a generated
+    /// request. This will use the raw header if it's present and a valid MIME
+    /// type, otherwise it will fall back to the content type of the body, if
+    /// known (e.g. JSON). Otherwise, return `None`. If the header is a
+    /// dynamic template, we will *not* attempt to render it, so MIME parsing
+    /// will fail.
+    pub fn mime(&self) -> Option<Mime> {
+        self.headers
+            .get(header::CONTENT_TYPE.as_str())
+            .and_then(|template| template.display().parse::<Mime>().ok())
+            .or_else(|| self.body.as_ref()?.mime())
+    }
 }
 
 #[cfg(any(test, feature = "test"))]
@@ -177,7 +216,7 @@ impl crate::test_util::Factory for Recipe {
         Self {
             id: RecipeId::factory(()),
             name: None,
-            method: Method::Get,
+            method: HttpMethod::Get,
             url: "http://localhost/url".into(),
             body: None,
             authentication: None,
@@ -198,30 +237,6 @@ impl crate::test_util::Factory<&str> for Recipe {
     }
 }
 
-/// A definition of how to make a request. This is *not* called `Request` in
-/// order to distinguish it from a single instance of an HTTP request. And it's
-/// not called `RequestTemplate` because the word "template" has a specific
-/// meaning related to string interpolation.
-#[derive(Debug, Serialize, Deserialize)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(deny_unknown_fields)]
-pub struct Recipe {
-    #[serde(skip)] // This will be auto-populated from the map key
-    pub id: RecipeId,
-    pub name: Option<String>,
-    /// *Not* a template string because the usefulness doesn't justify the
-    /// complexity. This gives the user an immediate error if the method is
-    /// wrong which is helpful.
-    pub method: Method,
-    pub url: Template,
-    pub body: Option<RecipeBody>,
-    pub authentication: Option<Authentication>,
-    #[serde(default, with = "cereal::serde_query_parameters")]
-    pub query: Vec<(String, Template)>,
-    #[serde(default)]
-    pub headers: IndexMap<String, Template>,
-}
-
 #[derive(
     Clone,
     Debug,
@@ -235,6 +250,8 @@ pub struct Recipe {
     Serialize,
     Deserialize,
 )]
+#[deref(forward)]
+#[serde(transparent)]
 pub struct RecipeId(String);
 
 #[cfg(any(test, feature = "test"))]
@@ -251,44 +268,6 @@ impl crate::test_util::Factory for RecipeId {
     }
 }
 
-/// HTTP method. This is duplicated from reqwest's Method so we can enforce
-/// the method is valid during deserialization. This is also generally more
-/// ergonomic at the cost of some flexibility.
-///
-/// The FromStr implementation will be case-insensitive
-#[derive(
-    Copy, Clone, Debug, Display, EnumIter, FromStr, Serialize, Deserialize,
-)]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(into = "String", try_from = "String")]
-pub enum Method {
-    #[display("CONNECT")]
-    Connect,
-    #[display("DELETE")]
-    Delete,
-    #[display("GET")]
-    Get,
-    #[display("HEAD")]
-    Head,
-    #[display("OPTIONS")]
-    Options,
-    #[display("PATCH")]
-    Patch,
-    #[display("POST")]
-    Post,
-    #[display("PUT")]
-    Put,
-    #[display("TRACE")]
-    Trace,
-}
-
-/// For serialization
-impl From<Method> for String {
-    fn from(method: Method) -> Self {
-        method.to_string()
-    }
-}
-
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory for Chain {
     fn factory(_: ()) -> Self {
@@ -356,6 +335,22 @@ impl RecipeBody {
             content_type: Some(ContentType::Json),
         }
     }
+
+    /// Get the anticipated MIME type that will appear in the `Content-Type`
+    /// header of a request containing this body. This is *not* necessarily
+    /// the MIME type that will _actually_ be used, as it could be overidden by
+    /// an explicit header.
+    pub fn mime(&self) -> Option<Mime> {
+        match self {
+            RecipeBody::Raw { content_type, .. } => {
+                content_type.as_ref().map(ContentType::to_mime)
+            }
+            RecipeBody::FormUrlencoded(_) => {
+                Some(mime::APPLICATION_WWW_FORM_URLENCODED)
+            }
+            RecipeBody::FormMultipart(_) => Some(mime::MULTIPART_FORM_DATA),
+        }
+    }
 }
 
 #[cfg(any(test, feature = "test"))]
@@ -412,6 +407,8 @@ pub struct Chain {
     Serialize,
     Deserialize,
 )]
+#[deref(forward)]
+#[serde(transparent)]
 #[cfg_attr(test, derive(proptest_derive::Arbitrary))]
 pub struct ChainId(#[deref(forward)] Identifier);
 
@@ -583,7 +580,14 @@ impl Collection {
 impl crate::test_util::Factory for Collection {
     fn factory(_: ()) -> Self {
         use crate::test_util::by_id;
-        let recipe = Recipe::factory(());
+        // Include a body in the recipe, so body-related behavior can be tested
+        let recipe = Recipe {
+            body: Some(RecipeBody::Raw {
+                body: r#"{"message": "hello"}"#.into(),
+                content_type: Some(ContentType::Json),
+            }),
+            ..Recipe::factory(())
+        };
         let profile = Profile::factory(());
         Collection {
             recipes: by_id([recipe]).into(),
@@ -593,17 +597,74 @@ impl crate::test_util::Factory for Collection {
     }
 }
 
-/// For deserialization
-impl TryFrom<String> for Method {
-    type Error = anyhow::Error;
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::test_util::Factory;
+    use indexmap::indexmap;
+    use rstest::rstest;
 
-    fn try_from(value: String) -> Result<Self, Self::Error> {
-        // Provide a better error than what's generated
-        value.parse().map_err(|_| {
-            anyhow!(
-                "Invalid HTTP method `{value}`. Must be one of: {}",
-                Method::iter().map(|method| method.to_string()).format(", ")
-            )
-        })
+    #[rstest]
+    #[case::none(None, None, None)]
+    #[case::header(
+        // Header takes precedence over body
+        Some("text/plain"),
+        Some(RecipeBody::Raw {
+            body: "hi!".into(),
+            content_type: Some(ContentType::Json),
+        }),
+        Some("text/plain")
+    )]
+    #[case::unknown_mime(
+        // Fall back to body type
+        Some("bogus"),
+        Some(RecipeBody::Raw {
+            body: "hi!".into(),
+            content_type: Some(ContentType::Json),
+        }),
+        Some("application/json")
+    )]
+    #[case::json_body(
+        None,
+        Some(RecipeBody::Raw {
+            body: "hi!".into(),
+            content_type: Some(ContentType::Json),
+        }),
+        Some("application/json")
+    )]
+    #[case::unknown_body(
+        None,
+        Some(RecipeBody::Raw {
+            body: "hi!".into(),
+            content_type: None,
+        }),
+        None,
+    )]
+    #[case::form_urlencoded_body(
+        None,
+        Some(RecipeBody::FormUrlencoded(indexmap! {})),
+        Some("application/x-www-form-urlencoded")
+    )]
+    #[case::form_multipart_body(
+        None,
+        Some(RecipeBody::FormMultipart(indexmap! {})),
+        Some("multipart/form-data")
+    )]
+    fn test_recipe_mime(
+        #[case] header: Option<&str>,
+        #[case] body: Option<RecipeBody>,
+        #[case] expected: Option<&str>,
+    ) {
+        let mut headers = IndexMap::new();
+        if let Some(header) = header {
+            headers.insert("content-type".into(), header.into());
+        }
+        let recipe = Recipe {
+            headers,
+            body,
+            ..Recipe::factory(())
+        };
+        let expected = expected.and_then(|value| value.parse::<Mime>().ok());
+        assert_eq!(recipe.mime(), expected);
     }
 }
