@@ -175,6 +175,7 @@ impl Database {
     pub fn into_collection(
         self,
         path: &Path,
+        mode: DatabaseMode,
     ) -> anyhow::Result<CollectionDatabase> {
         // Convert to canonicalize and make serializable
         let path: CollectionPath = path.try_into()?;
@@ -205,6 +206,7 @@ impl Database {
         Ok(CollectionDatabase {
             collection_id,
             database: self,
+            mode,
         })
     }
 }
@@ -216,6 +218,7 @@ impl Database {
 pub struct CollectionDatabase {
     collection_id: CollectionId,
     database: Database,
+    mode: DatabaseMode,
 }
 
 impl CollectionDatabase {
@@ -233,6 +236,20 @@ impl CollectionDatabase {
             .map(PathBuf::from)
     }
 
+    /// Is read/write mode enabled for this database?
+    pub fn can_write(&self) -> bool {
+        self.mode == DatabaseMode::ReadWrite
+    }
+
+    /// Return an error if we are in read-only mode
+    fn ensure_write(&self) -> anyhow::Result<()> {
+        if self.can_write() {
+            Ok(())
+        } else {
+            Err(anyhow!("Database in read-only mode"))
+        }
+    }
+
     /// Get a request by ID, or `None` if it does not exist in history.
     pub fn get_request(
         &self,
@@ -265,7 +282,7 @@ impl CollectionDatabase {
     /// profile is `None`, match all requests that have no associated profile.
     pub fn get_latest_request(
         &self,
-        profile_id: Option<&ProfileId>,
+        profile_id: ProfileFilter,
         recipe_id: &RecipeId,
     ) -> anyhow::Result<Option<Exchange>> {
         trace!(
@@ -276,14 +293,19 @@ impl CollectionDatabase {
         self.database
             .connection()
             .query_row(
-                // `IS` needed for profile_id so `None` will match `NULL`
+                // `IS` needed for profile_id so `None` will match `NULL`.
+                // We want to dynamically ignore the profile filter if the user
+                // is asking for all profiles. Dynamically modifying the query
+                // is really ugly so the easiest thing is to use an additional
+                // parameter to bypass the filter
                 "SELECT * FROM requests_v2
                 WHERE collection_id = :collection_id
-                    AND profile_id IS :profile_id
+                    AND (:ignore_profile_id OR profile_id IS :profile_id)
                     AND recipe_id = :recipe_id
                 ORDER BY start_time DESC LIMIT 1",
                 named_params! {
                     ":collection_id": self.collection_id,
+                    ":ignore_profile_id": profile_id == ProfileFilter::All,
                     ":profile_id": profile_id,
                     ":recipe_id": recipe_id,
                 },
@@ -292,21 +314,66 @@ impl CollectionDatabase {
             .optional()
             .with_context(|| {
                 format!(
-                    "Error fetching request [profile={}; recipe={}] \
+                    "Error fetching request [profile={:?}; recipe={}] \
                     from database",
-                    profile_id.map(ProfileId::to_string).unwrap_or_default(),
-                    recipe_id
+                    profile_id, recipe_id
                 )
             })
             .traced()
     }
 
+    /// Get a list of all requests for a profile+recipe combo
+    pub fn get_all_requests(
+        &self,
+        profile_id: ProfileFilter,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<Vec<ExchangeSummary>> {
+        trace!(
+            profile_id = ?profile_id,
+            recipe_id = %recipe_id,
+            "Fetching request history from database"
+        );
+        // It would be nice to de-dupe this code with get_all_requests, but
+        // there's no good way to dynamically build a query with sqlite so it
+        // ends up not being worth it
+        self.database
+            .connection()
+            .prepare(
+                // `IS` needed for profile_id so `None` will match `NULL`.
+                // We want to dynamically ignore the profile filter if the user
+                // is asking for all profiles. Dynamically modifying the query
+                // is really ugly so the easiest thing is to use an additional
+                // parameter to bypass the filter
+                "SELECT id, profile_id, start_time, end_time, status_code
+                FROM requests_v2
+                WHERE collection_id = :collection_id
+                    AND (:ignore_profile_id OR profile_id IS :profile_id)
+                    AND recipe_id = :recipe_id
+                ORDER BY start_time DESC",
+            )?
+            .query_map(
+                named_params! {
+                    ":collection_id": self.collection_id,
+                    ":ignore_profile_id": profile_id == ProfileFilter::All,
+                    ":profile_id": profile_id,
+                    ":recipe_id": recipe_id,
+                },
+                |row| row.try_into(),
+            )
+            .context("Error fetching request history from database")
+            .traced()?
+            .collect::<rusqlite::Result<Vec<_>>>()
+            .context("Error extracting request history")
+    }
+
     /// Add a new exchange to history. The HTTP engine is responsible for
     /// inserting its own exchanges. Only requests that received a valid HTTP
     /// response should be stored. In-flight requests, invalid requests, and
     /// requests that failed to complete (e.g. because of a network error)
     /// should not (and cannot) be stored.
     pub fn insert_exchange(&self, exchange: &Exchange) -> anyhow::Result<()> {
+        self.ensure_write()?;
+
         debug!(
             id = %exchange.id,
             url = %exchange.request.url,
@@ -323,6 +390,7 @@ impl CollectionDatabase {
                     recipe_id,
                     start_time,
                     end_time,
+                    http_version,
                     method,
                     url,
                     request_headers,
@@ -338,6 +406,7 @@ impl CollectionDatabase {
                     :recipe_id,
                     :start_time,
                     :end_time,
+                    :http_version,
                     :method,
                     :url,
                     :request_headers,
@@ -354,7 +423,8 @@ impl CollectionDatabase {
                     ":start_time": &exchange.start_time,
                     ":end_time": &exchange.end_time,
 
-                    ":method": exchange.request.method.as_str(),
+                    ":http_version": exchange.request.http_version,
+                    ":method": exchange.request.method,
                     ":url": exchange.request.url.as_str(),
                     ":request_headers": SqlWrap(&exchange.request.headers),
                     ":request_body": exchange.request.body(),
@@ -372,40 +442,6 @@ impl CollectionDatabase {
         Ok(())
     }
 
-    /// Get a list of all requests for a profile+recipe combo
-    pub fn get_all_requests(
-        &self,
-        profile_id: Option<&ProfileId>,
-        recipe_id: &RecipeId,
-    ) -> anyhow::Result<Vec<ExchangeSummary>> {
-        trace!(
-            profile_id = ?profile_id,
-            recipe_id = %recipe_id,
-            "Fetching request history from database"
-        );
-        self.database
-            .connection()
-            .prepare(
-                "SELECT id, start_time, end_time, status_code FROM requests_v2
-                WHERE collection_id = :collection_id
-                    AND profile_id IS :profile_id
-                    AND recipe_id = :recipe_id
-                ORDER BY start_time DESC",
-            )?
-            .query_map(
-                named_params! {
-                    ":collection_id": self.collection_id,
-                    ":profile_id": profile_id,
-                    ":recipe_id": recipe_id,
-                },
-                |row| row.try_into(),
-            )
-            .context("Error fetching request history from database")
-            .traced()?
-            .collect::<rusqlite::Result<Vec<_>>>()
-            .context("Error extracting request history")
-    }
-
     /// Get the value of a UI state field. Key type is included as part of the
     /// key, to disambiguate between keys of identical structure
     pub fn get_ui<K, V>(
@@ -453,6 +489,8 @@ impl CollectionDatabase {
         K: Debug + Serialize,
         V: Debug + Serialize,
     {
+        self.ensure_write()?;
+
         debug!(?key, ?value, "Setting UI state");
         self.database
             .connection()
@@ -507,17 +545,53 @@ impl crate::test_util::Factory for Database {
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory for CollectionDatabase {
     fn factory(_: ()) -> Self {
+        Self::factory(DatabaseMode::ReadWrite)
+    }
+}
+
+#[cfg(any(test, feature = "test"))]
+impl crate::test_util::Factory<DatabaseMode> for CollectionDatabase {
+    fn factory(mode: DatabaseMode) -> Self {
         use crate::util::paths::get_repo_root;
         Database::factory(())
-            .into_collection(&get_repo_root().join("slumber.yml"))
+            .into_collection(&get_repo_root().join("slumber.yml"), mode)
             .expect("Error initializing DB collection")
     }
 }
 
+/// Is the database read-only or read/write?
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub enum DatabaseMode {
+    ReadOnly,
+    ReadWrite,
+}
+
+/// Define how to filter requests by profile
+#[derive(Debug, Default, PartialEq)]
+pub enum ProfileFilter<'a> {
+    /// Show requests with _no_ associated profile
+    None,
+    /// Show requests for a particular profile
+    Some(&'a ProfileId),
+    /// Show requests for all profiles
+    #[default]
+    All,
+}
+
+/// Convert from an option that defines either *no* profile or a specific one
+impl<'a> From<Option<&'a ProfileId>> for ProfileFilter<'a> {
+    fn from(value: Option<&'a ProfileId>) -> Self {
+        match value {
+            Some(profile_id) => Self::Some(profile_id),
+            None => Self::None,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{test_util::Factory, util::paths::get_repo_root};
+    use crate::{assert_err, test_util::Factory, util::paths::get_repo_root};
     use itertools::Itertools;
     use std::collections::HashMap;
 
@@ -526,8 +600,14 @@ mod tests {
         let database = Database::factory(());
         let path1 = get_repo_root().join("slumber.yml");
         let path2 = get_repo_root().join("README.md"); // Has to be a real file
-        let collection1 = database.clone().into_collection(&path1).unwrap();
-        let collection2 = database.clone().into_collection(&path2).unwrap();
+        let collection1 = database
+            .clone()
+            .into_collection(&path1, DatabaseMode::ReadWrite)
+            .unwrap();
+        let collection2 = database
+            .clone()
+            .into_collection(&path2, DatabaseMode::ReadWrite)
+            .unwrap();
 
         let exchange1 =
             Exchange::factory((Some("profile1".into()), "recipe1".into()));
@@ -545,7 +625,7 @@ mod tests {
         // Sanity checks
         assert_eq!(
             collection1
-                .get_latest_request(profile_id, recipe_id)
+                .get_latest_request(profile_id.into(), recipe_id)
                 .unwrap()
                 .unwrap()
                 .id,
@@ -557,7 +637,7 @@ mod tests {
         );
         assert_eq!(
             collection2
-                .get_latest_request(profile_id, recipe_id)
+                .get_latest_request(profile_id.into(), recipe_id)
                 .unwrap()
                 .unwrap()
                 .id,
@@ -574,7 +654,7 @@ mod tests {
         // Collection 2 values should've overwritten
         assert_eq!(
             collection1
-                .get_latest_request(profile_id, recipe_id)
+                .get_latest_request(profile_id.into(), recipe_id)
                 .unwrap()
                 .unwrap()
                 .id,
@@ -598,11 +678,17 @@ mod tests {
         let database = Database::factory(());
         let collection1 = database
             .clone()
-            .into_collection(&get_repo_root().join("slumber.yml"))
+            .into_collection(
+                &get_repo_root().join("slumber.yml"),
+                DatabaseMode::ReadWrite,
+            )
             .unwrap();
         let collection2 = database
             .clone()
-            .into_collection(&get_repo_root().join("README.md"))
+            .into_collection(
+                &get_repo_root().join("README.md"),
+                DatabaseMode::ReadWrite,
+            )
             .unwrap();
 
         let exchange2 = Exchange::factory(());
@@ -650,7 +736,10 @@ mod tests {
                     // Leave the Option here so a non-match will trigger a handy
                     // assertion error
                     let exchange_id = collection
-                        .get_latest_request(profile_id.as_ref(), &recipe_id)
+                        .get_latest_request(
+                            profile_id.as_ref().into(),
+                            &recipe_id,
+                        )
                         .unwrap()
                         .map(|exchange| exchange.id);
                     let expected_id = request_ids.get(&(
@@ -711,7 +800,7 @@ mod tests {
                 // Leave the Option here so a non-match will trigger a handy
                 // assertion error
                 let ids = database
-                    .get_all_requests(profile_id.as_ref(), &recipe_id)
+                    .get_all_requests(profile_id.as_ref().into(), &recipe_id)
                     .unwrap()
                     .into_iter()
                     .map(|exchange| exchange.id)
@@ -728,6 +817,24 @@ mod tests {
                 );
             }
         }
+
+        // Load all requests for a recipe (across all profiles)
+        let recipe_id = "recipe1".into();
+        let ids = database
+            .get_all_requests(ProfileFilter::All, &recipe_id)
+            .unwrap()
+            .into_iter()
+            .map(|exchange| exchange.id)
+            .sorted()
+            .collect_vec();
+        let expected_ids = request_ids
+            .iter()
+            .filter(|((_, r), _)| r == &recipe_id)
+            .flat_map(|(_, request_ids)| request_ids)
+            .sorted()
+            .copied()
+            .collect_vec();
+        assert_eq!(ids, expected_ids)
     }
 
     /// Test UI state storage and retrieval
@@ -736,11 +843,14 @@ mod tests {
         let database = Database::factory(());
         let collection1 = database
             .clone()
-            .into_collection(Path::new("../../slumber.yml"))
+            .into_collection(
+                Path::new("../../slumber.yml"),
+                DatabaseMode::ReadWrite,
+            )
             .unwrap();
         let collection2 = database
             .clone()
-            .into_collection(Path::new("Cargo.toml"))
+            .into_collection(Path::new("Cargo.toml"), DatabaseMode::ReadWrite)
             .unwrap();
 
         let key_type = "MyKey";
@@ -757,4 +867,17 @@ mod tests {
             Some("value2".into())
         );
     }
+
+    #[test]
+    fn test_readonly_mode() {
+        let database = CollectionDatabase::factory(DatabaseMode::ReadOnly);
+        assert_err!(
+            database.insert_exchange(&Exchange::factory(())),
+            "Database in read-only mode"
+        );
+        assert_err!(
+            database.set_ui("MyKey", "key1", "value1"),
+            "Database in read-only mode"
+        );
+    }
 }
