@@ -35,7 +35,6 @@
 //! +----------+
 //! ```
 
-mod cereal;
 pub mod content_type;
 mod models;
 pub mod query;
@@ -43,7 +42,7 @@ pub mod query;
 pub use models::*;
 
 use crate::{
-    collection::{Authentication, Method, Recipe, RecipeBody},
+    collection::{Authentication, Recipe, RecipeBody},
     db::CollectionDatabase,
     http::content_type::ContentType,
     template::{Template, TemplateContext},
@@ -364,7 +363,7 @@ impl RequestTicket {
         let result = async {
             let response = self.client.execute(self.request).await?;
             // Load the full response and convert it to our format
-            ResponseRecord::from_response(response).await
+            ResponseRecord::from_response(id, response).await
         }
         .await;
         let end_time = Utc::now();
@@ -375,13 +374,15 @@ impl RequestTicket {
                 let exchange = Exchange {
                     id,
                     request: self.record,
-                    response,
+                    response: response.into(),
                     start_time,
                     end_time,
                 };
 
-                // Error here should *not* kill the request
-                let _ = database.insert_exchange(&exchange);
+                if database.can_write() {
+                    // Error here should *not* kill the request
+                    let _ = database.insert_exchange(&exchange);
+                }
                 Ok(exchange)
             }
 
@@ -403,6 +404,7 @@ impl ResponseRecord {
     /// because the response content is not necessarily loaded when we first get
     /// the response. Only fails if the response content fails to load.
     async fn from_response(
+        id: RequestId,
         response: Response,
     ) -> reqwest::Result<ResponseRecord> {
         // Copy response metadata out first, because we need to move the
@@ -414,6 +416,7 @@ impl ResponseRecord {
         let body = response.bytes().await?.into();
 
         Ok(ResponseRecord {
+            id,
             status,
             headers,
             body,
@@ -472,7 +475,7 @@ impl Recipe {
         // Set Content-Type based on the body type. This can be overwritten
         // below if the user explicitly passed a Content-Type value
         if let Some(content_type) =
-            self.body.as_ref().and_then(|body| body.mime())
+            self.body.as_ref().and_then(|body| body.explicit_mime())
         {
             headers.insert(
                 header::CONTENT_TYPE,
@@ -655,8 +658,10 @@ impl Authentication<String> {
 
 impl RecipeBody {
     /// Get the value that we should set for the `Content-Type` header,
-    /// according to the body
-    fn mime(&self) -> Option<Mime> {
+    /// according to the body. This will only return `Some` for JSON, as the
+    /// form content types will have this header set automatically by reqwest
+    /// via the builder methods we use.
+    fn explicit_mime(&self) -> Option<Mime> {
         match self {
             RecipeBody::Raw { content_type, .. } => {
                 content_type.as_ref().map(ContentType::to_mime)
@@ -700,18 +705,18 @@ impl RenderedBody {
     }
 }
 
-impl From<Method> for reqwest::Method {
-    fn from(method: Method) -> Self {
+impl From<HttpMethod> for reqwest::Method {
+    fn from(method: HttpMethod) -> Self {
         match method {
-            Method::Connect => reqwest::Method::CONNECT,
-            Method::Delete => reqwest::Method::DELETE,
-            Method::Get => reqwest::Method::GET,
-            Method::Head => reqwest::Method::HEAD,
-            Method::Options => reqwest::Method::OPTIONS,
-            Method::Patch => reqwest::Method::PATCH,
-            Method::Post => reqwest::Method::POST,
-            Method::Put => reqwest::Method::PUT,
-            Method::Trace => reqwest::Method::TRACE,
+            HttpMethod::Connect => reqwest::Method::CONNECT,
+            HttpMethod::Delete => reqwest::Method::DELETE,
+            HttpMethod::Get => reqwest::Method::GET,
+            HttpMethod::Head => reqwest::Method::HEAD,
+            HttpMethod::Options => reqwest::Method::OPTIONS,
+            HttpMethod::Patch => reqwest::Method::PATCH,
+            HttpMethod::Post => reqwest::Method::POST,
+            HttpMethod::Put => reqwest::Method::PUT,
+            HttpMethod::Trace => reqwest::Method::TRACE,
         }
     }
 }
@@ -741,9 +746,7 @@ fn trim_bytes(bytes: &mut Vec<u8>, f: impl Fn(u8) -> bool) {
 mod tests {
     use super::*;
     use crate::{
-        collection::{
-            self, Authentication, Chain, ChainSource, Collection, Profile,
-        },
+        collection::{Authentication, Chain, ChainSource, Collection, Profile},
         test_util::{
             by_id, header_map, http_engine, invalid_utf8_chain, Factory,
             TestPrompter,
@@ -752,7 +755,7 @@ mod tests {
     use indexmap::{indexmap, IndexMap};
     use pretty_assertions::assert_eq;
     use regex::Regex;
-    use reqwest::{Body, Method, StatusCode};
+    use reqwest::{Body, StatusCode};
     use rstest::rstest;
     use serde_json::json;
     use std::ptr;
@@ -830,7 +833,7 @@ mod tests {
     #[tokio::test]
     async fn test_build_request(http_engine: &HttpEngine) {
         let recipe = Recipe {
-            method: collection::Method::Post,
+            method: HttpMethod::Post,
             url: "{{host}}/users/{{user_id}}".into(),
             query: vec![
                 ("mode".into(), "{{mode}}".into()),
@@ -861,7 +864,7 @@ mod tests {
 
         // Assert on the actual request
         let request = &ticket.request;
-        assert_eq!(request.method(), Method::POST);
+        assert_eq!(request.method(), reqwest::Method::POST);
         assert_eq!(request.url(), &expected_url);
         assert_eq!(request.headers(), &expected_headers);
         assert_eq!(
@@ -878,7 +881,8 @@ mod tests {
                     template_context.collection.first_profile_id().clone()
                 ),
                 recipe_id,
-                method: Method::POST,
+                method: HttpMethod::Post,
+                http_version: HttpVersion::Http11,
                 url: expected_url,
                 body: Some(Vec::from(expected_body).into()),
                 headers: expected_headers,
@@ -1015,7 +1019,8 @@ mod tests {
                     template_context.collection.first_profile_id().clone()
                 ),
                 recipe_id,
-                method: Method::GET,
+                method: HttpMethod::Get,
+                http_version: HttpVersion::Http11,
                 url: "http://localhost/url".parse().unwrap(),
                 headers: header_map([
                     ("authorization", "bogus"),
@@ -1231,7 +1236,8 @@ mod tests {
                 id: ticket.record.id,
                 profile_id: template_context.selected_profile.clone(),
                 recipe_id,
-                method: Method::GET,
+                method: HttpMethod::Get,
+                http_version: HttpVersion::Http11,
                 url: "http://localhost/url?mode=sudo&fast=true"
                     .parse()
                     .unwrap(),
@@ -1288,7 +1294,8 @@ mod tests {
                 id: ticket.record.id,
                 profile_id: template_context.selected_profile.clone(),
                 recipe_id,
-                method: Method::GET,
+                method: HttpMethod::Get,
+                http_version: HttpVersion::Http11,
                 url: "http://localhost/url".parse().unwrap(),
                 headers: header_map([(
                     "content-type",
@@ -1305,7 +1312,7 @@ mod tests {
     #[tokio::test]
     async fn test_chain_duplicate(http_engine: &HttpEngine) {
         let recipe = Recipe {
-            method: collection::Method::Post,
+            method: HttpMethod::Post,
             url: "{{host}}/{{chains.text}}".into(),
             body: Some("{{chains.text}}".into()),
             ..Recipe::factory(())
@@ -1362,8 +1369,9 @@ mod tests {
             .to_str()
             .unwrap();
         assert_eq!(
-            exchange.response,
+            *exchange.response,
             ResponseRecord {
+                id: exchange.id,
                 status: StatusCode::OK,
                 headers: header_map([
                     ("content-type", "text/plain"),
