@@ -5,20 +5,18 @@
 
 use crate::{
     collection::{Authentication, ProfileId, RecipeBody, RecipeId},
-    http::{
-        cereal,
-        content_type::{ContentType, ResponseContent},
-    },
+    http::content_type::ContentType,
     template::Template,
 };
 use anyhow::Context;
 use bytes::Bytes;
 use chrono::{DateTime, Duration, Utc};
 use derive_more::{Display, From, FromStr};
+use itertools::Itertools;
 use mime::Mime;
 use reqwest::{
     header::{self, HeaderMap},
-    Body, Client, Method, Request, StatusCode, Url,
+    Body, Client, Request, StatusCode, Url,
 };
 use serde::{Deserialize, Serialize};
 use std::{
@@ -26,6 +24,7 @@ use std::{
     fmt::{Debug, Write},
     sync::Arc,
 };
+use strum::{EnumIter, IntoEnumIterator};
 use thiserror::Error;
 use tracing::error;
 use uuid::Uuid;
@@ -44,6 +43,7 @@ use uuid::Uuid;
     Serialize,
     Deserialize,
 )]
+#[cfg_attr(test, derive(PartialOrd, Ord))]
 pub struct RequestId(pub Uuid);
 
 impl RequestId {
@@ -58,6 +58,190 @@ impl Default for RequestId {
     }
 }
 
+/// HTTP protocl version. This is duplicated from [reqwest::Version] because
+/// that type doesn't provide any way to construct it. It only allows you to use
+/// the existing constants.
+#[derive(Copy, Clone, Debug, Default, EnumIter, Serialize, Deserialize)]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(into = "&str", try_from = "String")]
+pub enum HttpVersion {
+    Http09,
+    Http10,
+    #[default]
+    Http11,
+    Http2,
+    Http3,
+}
+
+impl HttpVersion {
+    pub fn to_str(self) -> &'static str {
+        match self {
+            Self::Http09 => "HTTP/0.9",
+            Self::Http10 => "HTTP/1.0",
+            Self::Http11 => "HTTP/1.1",
+            Self::Http2 => "HTTP/2.0",
+            Self::Http3 => "HTTP/3.0",
+        }
+    }
+}
+
+impl Display for HttpVersion {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(self.to_str())
+    }
+}
+
+impl From<reqwest::Version> for HttpVersion {
+    fn from(version: reqwest::Version) -> Self {
+        match version {
+            reqwest::Version::HTTP_09 => Self::Http09,
+            reqwest::Version::HTTP_10 => Self::Http10,
+            reqwest::Version::HTTP_11 => Self::Http11,
+            reqwest::Version::HTTP_2 => Self::Http2,
+            reqwest::Version::HTTP_3 => Self::Http3,
+            _ => panic!("Unrecognized HTTP version: {version:?}"),
+        }
+    }
+}
+
+impl FromStr for HttpVersion {
+    type Err = HttpVersionParseError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_uppercase().as_str() {
+            "HTTP/0.9" => Ok(Self::Http09),
+            "HTTP/1.0" => Ok(Self::Http10),
+            "HTTP/1.1" => Ok(Self::Http11),
+            "HTTP/2.0" => Ok(Self::Http2),
+            "HTTP/3.0" => Ok(Self::Http3),
+            _ => Err(HttpVersionParseError {
+                input: s.to_owned(),
+            }),
+        }
+    }
+}
+
+/// For serialization
+impl From<HttpVersion> for &'static str {
+    fn from(version: HttpVersion) -> Self {
+        version.to_str()
+    }
+}
+
+/// For deserialization
+impl TryFrom<String> for HttpVersion {
+    type Error = <Self as FromStr>::Err;
+
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        value.parse()
+    }
+}
+
+#[derive(Debug, Error)]
+#[error(
+    "Invalid HTTP version `{input}`. Must be one of: {}",
+    HttpVersion::iter().map(|method| method.to_str()).format(", "),
+)]
+pub struct HttpVersionParseError {
+    input: String,
+}
+
+/// HTTP method. This is duplicated from [reqwest::Method] so we can enforce
+/// the method is valid during deserialization. This is also generally more
+/// ergonomic at the cost of some flexibility.
+///
+/// The FromStr implementation will be case-insensitive
+#[derive(Copy, Clone, Debug, EnumIter, Serialize, Deserialize)]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(into = "&str", try_from = "String")]
+pub enum HttpMethod {
+    Connect,
+    Delete,
+    Get,
+    Head,
+    Options,
+    Patch,
+    Post,
+    Put,
+    Trace,
+}
+
+impl HttpMethod {
+    pub fn to_str(self) -> &'static str {
+        match self {
+            Self::Connect => "CONNECT",
+            Self::Delete => "DELETE",
+            Self::Get => "GET",
+            Self::Head => "HEAD",
+            Self::Options => "OPTIONS",
+            Self::Patch => "PATCH",
+            Self::Post => "POST",
+            Self::Put => "PUT",
+            Self::Trace => "TRACE",
+        }
+    }
+}
+
+impl Display for HttpMethod {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(self.to_str())
+    }
+}
+
+impl FromStr for HttpMethod {
+    type Err = HttpMethodParseError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_ascii_uppercase().as_str() {
+            "CONNECT" => Ok(Self::Connect),
+            "DELETE" => Ok(Self::Delete),
+            "GET" => Ok(Self::Get),
+            "HEAD" => Ok(Self::Head),
+            "OPTIONS" => Ok(Self::Options),
+            "PATCH" => Ok(Self::Patch),
+            "POST" => Ok(Self::Post),
+            "PUT" => Ok(Self::Put),
+            "TRACE" => Ok(Self::Trace),
+            _ => Err(HttpMethodParseError {
+                input: s.to_owned(),
+            }),
+        }
+    }
+}
+
+impl From<&reqwest::Method> for HttpMethod {
+    fn from(method: &reqwest::Method) -> Self {
+        // reqwest supports custom methods, but we don't provide any
+        // mechanism for users to use them, so we should never panic
+        method.as_str().parse().unwrap()
+    }
+}
+
+/// For serialization
+impl From<HttpMethod> for &'static str {
+    fn from(method: HttpMethod) -> Self {
+        method.to_str()
+    }
+}
+
+/// For deserialization
+impl TryFrom<String> for HttpMethod {
+    type Error = <Self as FromStr>::Err;
+
+    fn try_from(method: String) -> Result<Self, Self::Error> {
+        method.parse()
+    }
+}
+
+#[derive(Debug, Error)]
+#[error(
+    "Invalid HTTP method `{input}`. Must be one of: {}",
+    HttpMethod::iter().map(|method| method.to_str()).format(", "),
+)]
+pub struct HttpMethodParseError {
+    input: String,
+}
+
 /// The first stage in building a request. This contains the initialization data
 /// needed to build a request. This holds owned data because we need to be able
 /// to move it between tasks as part of the build process, which requires it
@@ -183,7 +367,7 @@ pub struct Exchange {
     /// What we said. Use an Arc so the view can hang onto it.
     pub request: Arc<RequestRecord>,
     /// What we heard
-    pub response: ResponseRecord,
+    pub response: Arc<ResponseRecord>,
     /// When was the request sent to the server?
     pub start_time: DateTime<Utc>,
     /// When did we finish receiving the *entire* response?
@@ -199,9 +383,10 @@ impl Exchange {
 
 /// Metadata about an exchange. Useful in lists where request/response content
 /// isn't needed.
-#[derive(Copy, Clone, Debug)]
+#[derive(Clone, Debug)]
 pub struct ExchangeSummary {
     pub id: RequestId,
+    pub profile_id: Option<ProfileId>,
     pub start_time: DateTime<Utc>,
     pub end_time: DateTime<Utc>,
     pub status: StatusCode,
@@ -211,6 +396,7 @@ impl From<&Exchange> for ExchangeSummary {
     fn from(exchange: &Exchange) -> Self {
         Self {
             id: exchange.id,
+            profile_id: exchange.request.profile_id.clone(),
             start_time: exchange.start_time,
             end_time: exchange.end_time,
             status: exchange.response.status,
@@ -228,9 +414,7 @@ impl From<&Exchange> for ExchangeSummary {
 /// This intentionally does *not* implement `Clone`, because request data could
 /// potentially be large so we want to be intentional about duplicating it only
 /// when necessary.
-///
-/// Remove serde impls in https://github.com/LucasPickering/slumber/issues/306
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
 pub struct RequestRecord {
     /// Unique ID for this request
@@ -240,11 +424,13 @@ pub struct RequestRecord {
     /// The recipe used to generate this request (for historical context)
     pub recipe_id: RecipeId,
 
-    #[serde(with = "cereal::serde_method")]
-    pub method: Method,
+    /// HTTP protocol version. Unlike `method`, we can't use the reqwest type
+    /// here because there's way to externally construct the type.
+    pub http_version: HttpVersion,
+    /// HTTP method
+    pub method: HttpMethod,
     /// URL, including query params/fragment
     pub url: Url,
-    #[serde(with = "cereal::serde_header_map")]
     pub headers: HeaderMap,
     /// Body content as bytes. This should be decoded as needed. This will
     /// **not** be populated for bodies that are above the "large" threshold.
@@ -272,7 +458,8 @@ impl RequestRecord {
             profile_id,
             recipe_id: seed.recipe_id,
 
-            method: request.method().clone(),
+            http_version: request.version().into(),
+            method: request.method().into(),
             url: request.url().clone(),
             headers: request.headers().clone(),
             body: request
@@ -287,6 +474,11 @@ impl RequestRecord {
         }
     }
 
+    /// Get the value of the request's `Content-Type` header, if any
+    pub fn mime(&self) -> Option<Mime> {
+        content_type_header(&self.headers)
+    }
+
     /// Generate a cURL command equivalent to this request
     ///
     /// This only fails if one of the headers or body is binary and can't be
@@ -333,14 +525,16 @@ impl RequestRecord {
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory for RequestRecord {
     fn factory(_: ()) -> Self {
+        Self::factory(RequestId::new())
+    }
+}
+
+#[cfg(any(test, feature = "test"))]
+impl crate::test_util::Factory<RequestId> for RequestRecord {
+    fn factory(id: RequestId) -> Self {
         Self {
-            id: RequestId::new(),
-            profile_id: None,
-            recipe_id: RecipeId::factory(()),
-            method: reqwest::Method::GET,
-            url: "http://localhost/url".parse().unwrap(),
-            headers: HeaderMap::new(),
-            body: None,
+            id,
+            ..Self::factory((None, RecipeId::factory(())))
         }
     }
 }
@@ -356,7 +550,8 @@ impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)>
             id: RequestId::new(),
             profile_id,
             recipe_id,
-            method: reqwest::Method::GET,
+            method: HttpMethod::Get,
+            http_version: HttpVersion::Http11,
             url: "http://localhost/url".parse().unwrap(),
             headers: header_map([
                 ("Accept", "application/json"),
@@ -371,7 +566,15 @@ impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)>
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory for ResponseRecord {
     fn factory(_: ()) -> Self {
+        Self::factory(RequestId::new())
+    }
+}
+
+#[cfg(any(test, feature = "test"))]
+impl crate::test_util::Factory<RequestId> for ResponseRecord {
+    fn factory(id: RequestId) -> Self {
         Self {
+            id,
             status: StatusCode::OK,
             headers: HeaderMap::new(),
             body: ResponseBody::default(),
@@ -383,6 +586,7 @@ impl crate::test_util::Factory for ResponseRecord {
 impl crate::test_util::Factory<StatusCode> for ResponseRecord {
     fn factory(status: StatusCode) -> Self {
         Self {
+            id: RequestId::new(),
             status,
             headers: HeaderMap::new(),
             body: ResponseBody::default(),
@@ -409,27 +613,44 @@ impl crate::test_util::Factory<RecipeId> for Exchange {
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory<(Option<ProfileId>, RecipeId)> for Exchange {
     fn factory(params: (Option<ProfileId>, RecipeId)) -> Self {
+        let id = RequestId::new();
         Self::factory((
-            RequestRecord::factory(params),
-            ResponseRecord::factory(()),
+            RequestRecord {
+                id,
+                ..RequestRecord::factory(params)
+            },
+            ResponseRecord::factory(id),
         ))
     }
 }
 
-/// Customize profile and recipe ID
+/// Custom request and response
 #[cfg(any(test, feature = "test"))]
 impl crate::test_util::Factory<(RequestRecord, ResponseRecord)> for Exchange {
     fn factory((request, response): (RequestRecord, ResponseRecord)) -> Self {
+        // Request and response should've been generated from the same ID,
+        // otherwise we're going to see some shitty bugs
+        assert_eq!(
+            request.id, response.id,
+            "Request and response have different IDs"
+        );
         Self {
             id: request.id,
             request: request.into(),
-            response,
+            response: response.into(),
             start_time: Utc::now(),
             end_time: Utc::now(),
         }
     }
 }
 
+#[cfg(any(test, feature = "test"))]
+impl crate::test_util::Factory<RequestId> for Exchange {
+    fn factory(id: RequestId) -> Self {
+        Self::factory((RequestRecord::factory(id), ResponseRecord::factory(id)))
+    }
+}
+
 /// A resolved HTTP response, with all content loaded and ready to be displayed
 /// to the user. A simpler alternative to [reqwest::Response], because there's
 /// no way to access all resolved data on that type at once. Resolving the
@@ -437,33 +658,25 @@ impl crate::test_util::Factory<(RequestRecord, ResponseRecord)> for Exchange {
 ///
 /// This intentionally does not implement Clone, because responses could
 /// potentially be very large.
-///
-/// Remove serde impls in https://github.com/LucasPickering/slumber/issues/306
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
 pub struct ResponseRecord {
-    #[serde(with = "cereal::serde_status_code")]
+    pub id: RequestId,
     pub status: StatusCode,
-    #[serde(with = "cereal::serde_header_map")]
     pub headers: HeaderMap,
     pub body: ResponseBody,
 }
 
 impl ResponseRecord {
-    /// Stored the parsed form of this request's body
-    pub fn set_parsed_body(&mut self, body: Box<dyn ResponseContent>) {
-        self.body.parsed = Some(body);
+    /// Get the value of the response's `Content-Type` header, if any
+    pub fn mime(&self) -> Option<Mime> {
+        content_type_header(&self.headers)
     }
 
-    /// Get the content type of the response body, according to the
-    /// `Content-Type` header
+    /// Get the value of the response's `Content-Type` header, and parse it as
+    /// a known/supported content type
     pub fn content_type(&self) -> Option<ContentType> {
-        // If we've parsed the body, we'll have the content type present. If
-        // not, check the header now
-        self.body
-            .parsed()
-            .map(|content| content.content_type())
-            .or_else(|| ContentType::from_headers(&self.headers).ok())
+        ContentType::from_headers(&self.headers).ok()
     }
 
     /// Get a suggested file name for the content of this response. First we'll
@@ -496,64 +709,48 @@ impl ResponseRecord {
     }
 }
 
-pub enum ParseMode {
-    Immediate,
-    Background {
-        callback: Box<dyn 'static + FnOnce(Box<dyn ResponseContent>) + Send>,
-    },
+/// Get the value of the `Content-Type` header, parsed as a MIME. `None` if the
+/// header isn't present or isn't a valid MIME type
+fn content_type_header(headers: &HeaderMap) -> Option<Mime> {
+    headers
+        .get(header::CONTENT_TYPE)
+        .and_then(|value| value.to_str().ok()?.parse().ok())
 }
 
 /// HTTP response body. Content is stored as bytes because it may not
 /// necessarily be valid UTF-8. Converted to text only as needed.
-#[derive(Default, Deserialize)]
-#[serde(from = "Bytes")] // Can't use into=Bytes because that requires cloning
-pub struct ResponseBody {
+///
+/// The generic type is to make this usable with references to bodies. In most
+/// cases you can just use the default.
+#[derive(Default)]
+pub struct ResponseBody<T = Bytes> {
     /// Raw body
-    data: Bytes,
-    /// For responses of a known content type, we can parse the body into a
-    /// real data structure. This is populated manually; Call
-    /// [ResponseRecord::parse_body] to set the parsed body. This uses a lock
-    /// so it can be parsed and populated in a background thread.
-    #[serde(skip)]
-    parsed: Option<Box<dyn ResponseContent>>,
+    data: T,
 }
 
-impl ResponseBody {
-    pub fn new(data: Bytes) -> Self {
-        Self {
-            data,
-            parsed: Default::default(),
-        }
+impl<T: AsRef<[u8]>> ResponseBody<T> {
+    pub fn new(data: T) -> Self {
+        Self { data }
     }
 
     /// Raw content bytes
-    pub fn bytes(&self) -> &Bytes {
+    pub fn bytes(&self) -> &T {
         &self.data
     }
 
     /// Owned raw content bytes
-    pub fn into_bytes(self) -> Bytes {
+    pub fn into_bytes(self) -> T {
         self.data
     }
 
     /// Get bytes as text, if valid UTF-8
     pub fn text(&self) -> Option<&str> {
-        std::str::from_utf8(&self.data).ok()
+        std::str::from_utf8(self.data.as_ref()).ok()
     }
 
     /// Get body size, in bytes
     pub fn size(&self) -> usize {
-        self.bytes().len()
-    }
-
-    /// Get the parsed version of this body. Must haved call
-    /// [ResponseRecord::parse_body] first to actually do the parse. Parsing has
-    /// to be done on the parent because we don't have access to the
-    /// `Content-Type` header here, which tells us how to parse.
-    ///
-    /// Return `None` if parsing either hasn't happened yet, or failed.
-    pub fn parsed(&self) -> Option<&dyn ResponseContent> {
-        self.parsed.as_deref()
+        self.data.as_ref().len()
     }
 }
 
@@ -566,43 +763,27 @@ impl Debug for ResponseBody {
     }
 }
 
-impl From<Bytes> for ResponseBody {
-    fn from(bytes: Bytes) -> Self {
-        Self::new(bytes)
+impl<T: From<Bytes>> From<Bytes> for ResponseBody<T> {
+    fn from(data: Bytes) -> Self {
+        Self { data: data.into() }
     }
 }
 
-impl Serialize for ResponseBody {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: serde::Serializer,
-    {
-        // Serialize just the bytes, everything else is derived
-        self.data.serialize(serializer)
-    }
-}
-
-impl From<Vec<u8>> for ResponseBody {
-    fn from(value: Vec<u8>) -> Self {
-        Self::new(value.into())
-    }
-}
-
-#[cfg(test)]
+#[cfg(any(test, feature = "test"))]
 impl From<&str> for ResponseBody {
     fn from(value: &str) -> Self {
         Self::new(value.to_owned().into())
     }
 }
 
-#[cfg(test)]
+#[cfg(any(test, feature = "test"))]
 impl From<&[u8]> for ResponseBody {
     fn from(value: &[u8]) -> Self {
         Self::new(value.to_owned().into())
     }
 }
 
-#[cfg(test)]
+#[cfg(any(test, feature = "test"))]
 impl From<serde_json::Value> for ResponseBody {
     fn from(value: serde_json::Value) -> Self {
         Self::new(value.to_string().into())
@@ -737,7 +918,7 @@ mod tests {
         };
         let body = json!({"data": "value"});
         let request = RequestRecord {
-            method: Method::DELETE,
+            method: HttpMethod::Delete,
             headers: header_map(headers),
             body: Some(serde_json::to_vec(&body).unwrap().into()),
             ..RequestRecord::factory(())
