@@ -139,6 +139,7 @@ impl From<serde_json::Value> for Template {
     Serialize,
     Deserialize,
 )]
+#[serde(transparent)]
 #[cfg_attr(test, derive(proptest_derive::Arbitrary))]
 pub struct Identifier(
     #[cfg_attr(test, proptest(regex = "[a-zA-Z0-9-_]+"))] String,
@@ -440,7 +441,8 @@ mod tests {
             ..RequestRecord::factory(())
         };
         let response = ResponseRecord {
-            body: response_body.to_string().into_bytes().into(),
+            id: request.id,
+            body: response_body.into(),
             headers: response_headers,
             ..ResponseRecord::factory(())
         };
@@ -538,7 +540,7 @@ mod tests {
             response: ResponseRecord {
                 body: "not json!".into(),
                 ..ResponseRecord::factory(())
-            },
+            }.into(),
             ..Exchange::factory(RecipeId::from("recipe1"))
         }),
         "content type not provided",
@@ -560,7 +562,7 @@ mod tests {
             response: ResponseRecord {
                 body: "not json!".into(),
                 ..ResponseRecord::factory(())
-            },
+            }.into(),
             ..Exchange::factory(RecipeId::from("recipe1"))
         }),
         "Parsing response: expected ident at line 1 column 2",
@@ -582,7 +584,7 @@ mod tests {
             response: ResponseRecord {
                 body: "[1, 2]".into(),
                 ..ResponseRecord::factory(())
-            },
+            }.into(),
             ..Exchange::factory(RecipeId::from("recipe1"))
         }),
         "No results from JSONPath query",
@@ -1042,7 +1044,8 @@ mod tests {
             ..RequestRecord::factory(())
         };
         let response = ResponseRecord {
-            body: json!(["foo", "bar"]).to_string().into_bytes().into(),
+            id: request.id,
+            body: json!(["foo", "bar"]).into(),
             headers: response_headers,
             ..ResponseRecord::factory(())
         };
