@@ -18,11 +18,12 @@ mod view;
 
 use crate::{
     context::TuiContext,
-    http::{BackgroundResponseParser, RequestState, RequestStore},
-    message::{Message, MessageSender, RequestConfig},
+    http::{RequestState, RequestStore},
+    message::{Callback, Message, MessageSender, RequestConfig},
     util::{
         clear_event_buffer, delete_temp_file, get_editor_command,
-        get_viewer_command, save_file, signals, ResultReported,
+        get_pager_command, save_file, signals, spawn, spawn_result,
+        ResultReported, CANCEL_TOKEN,
     },
     view::{PreviewPrompter, UpdateContext, View},
 };
@@ -38,12 +39,11 @@ use ratatui::{prelude::CrosstermBackend, Terminal};
 use slumber_config::{Action, Config};
 use slumber_core::{
     collection::{Collection, CollectionFile, ProfileId},
-    db::{CollectionDatabase, Database},
+    db::{CollectionDatabase, Database, DatabaseMode},
     http::{RequestId, RequestSeed},
     template::{Prompter, Template, TemplateChunk, TemplateContext},
 };
 use std::{
-    future::Future,
     io::{self, Stdout},
     ops::Deref,
     path::PathBuf,
@@ -54,7 +54,7 @@ use std::{
 use tokio::{
     select,
     sync::mpsc::{self, UnboundedReceiver},
-    time,
+    task, time,
 };
 use tracing::{debug, error, info, info_span, trace};
 
@@ -87,7 +87,6 @@ impl Tui {
     /// Start the TUI. Any errors that occur during startup will be panics,
     /// because they prevent TUI execution.
     pub async fn start(collection_path: Option<PathBuf>) -> anyhow::Result<()> {
-        initialize_panic_handler();
         let collection_path = CollectionFile::try_path(None, collection_path)?;
 
         // ===== Initialize global state =====
@@ -101,7 +100,8 @@ impl Tui {
         // to default, just show an error to the user
         let config = Config::load().reported(&messages_tx).unwrap_or_default();
         // Load a database for this particular collection
-        let database = Database::load()?.into_collection(&collection_path)?;
+        let database = Database::load()?
+            .into_collection(&collection_path, DatabaseMode::ReadWrite)?;
         // Initialize global view context
         TuiContext::init(config);
 
@@ -113,19 +113,19 @@ impl Tui {
             .await
             .reported(&messages_tx)
             .unwrap_or_else(|| CollectionFile::with_path(collection_path));
-        let view =
-            View::new(&collection_file, database.clone(), messages_tx.clone());
+        let request_store = RequestStore::new(database.clone());
+        let view = View::new(
+            &collection_file,
+            &request_store,
+            database.clone(),
+            messages_tx.clone(),
+        );
 
         // The code to revert the terminal takeover is in `Tui::drop`, so we
         // shouldn't take over the terminal until right before creating the
         // `Tui`.
         let terminal = initialize_terminal()?;
 
-        let request_store = RequestStore::new(
-            database.clone(),
-            BackgroundResponseParser::new(messages_tx.clone()),
-        );
-
         let app = Tui {
             terminal,
             database,
@@ -139,7 +139,11 @@ impl Tui {
             request_store,
         };
 
-        app.run().await
+        // Run everything in one local set, so that we can use !Send values
+        let local = task::LocalSet::new();
+        local.spawn_local(app.run());
+        local.await;
+        Ok(())
     }
 
     /// Run the main TUI update loop. Any error returned from this is fatal. See
@@ -226,7 +230,7 @@ impl Tui {
             Message::CollectionStartReload => {
                 let future = self.collection_file.reload();
                 let messages_tx = self.messages_tx();
-                self.spawn(async move {
+                spawn_result(async move {
                     let collection = future.await?;
                     messages_tx.send(Message::CollectionEndReload(collection));
                     Ok(())
@@ -241,14 +245,14 @@ impl Tui {
                 self.run_command(command)?;
             }
 
-            Message::CopyRequestUrl(request_config) => {
-                self.copy_request_url(request_config)?;
+            Message::CopyRequestUrl => {
+                self.copy_request_url()?;
             }
-            Message::CopyRequestBody(request_config) => {
-                self.copy_request_body(request_config)?;
+            Message::CopyRequestBody => {
+                self.copy_request_body()?;
             }
-            Message::CopyRequestCurl(request_config) => {
-                self.copy_request_curl(request_config)?;
+            Message::CopyRequestCurl => {
+                self.copy_request_curl()?;
             }
             Message::CopyText(text) => self.view.copy_text(text),
             Message::SaveResponseBody { request_id, data } => {
@@ -262,15 +266,15 @@ impl Tui {
                 )?;
             }
 
-            Message::EditFile { path, on_complete } => {
+            Message::FileEdit { path, on_complete } => {
                 let command = get_editor_command(&path)?;
                 self.run_command(command)?;
                 on_complete(path);
                 // The callback may queue an event to read the file, so we can't
                 // delete it yet. Caller is responsible for cleaning up
             }
-            Message::ViewFile { path } => {
-                let command = get_viewer_command(&path)?;
+            Message::FileView { path, mime } => {
+                let command = get_pager_command(&path, mime.as_ref())?;
                 self.run_command(command)?;
                 // We don't need to read the contents back so we can clean up
                 delete_temp_file(&path);
@@ -279,21 +283,25 @@ impl Tui {
             Message::Error { error } => self.view.open_modal(error),
 
             // Manage HTTP life cycle
-            Message::HttpBeginRequest(request_config) => {
-                self.send_request(request_config)?
-            }
+            Message::HttpBeginRequest => self.send_request()?,
             Message::HttpBuildError { error } => {
-                self.request_store.build_error(error);
+                let state = self.request_store.build_error(error);
+                self.view.update_request(state);
             }
             Message::HttpLoading { request } => {
-                self.request_store.loading(request);
+                let state = self.request_store.loading(request);
+                self.view.update_request(state);
+            }
+            Message::HttpComplete(result) => {
+                let state = match result {
+                    Ok(exchange) => self.request_store.response(exchange),
+                    Err(error) => self.request_store.request_error(error),
+                };
+                self.view.update_request(state);
             }
-            Message::HttpComplete(result) => match result {
-                Ok(exchange) => self.request_store.response(exchange),
-                Err(error) => self.request_store.request_error(error),
-            },
             Message::HttpCancel(request_id) => {
-                self.request_store.cancel(request_id)
+                let state = self.request_store.cancel(request_id);
+                self.view.update_request(state);
             }
 
             // Force quit short-circuits the view/message cycle, to make sure
@@ -327,17 +335,6 @@ impl Tui {
                 self.view.open_modal(confirm);
             }
 
-            Message::ParseResponseBodyComplete { request_id, body } => {
-                self.request_store
-                    .set_parsed_body(request_id, body)
-                    .with_context(|| {
-                        format!(
-                            "Error storing parsed response body \
-                            for request {request_id}"
-                        )
-                    })?;
-            }
-
             Message::TemplatePreview {
                 template,
                 on_complete,
@@ -353,8 +350,9 @@ impl Tui {
                     on_complete,
                 )?;
             }
+
             // This message exists just to trigger a draw
-            Message::TemplatePreviewComplete => {}
+            Message::Tick => {}
 
             Message::Quit => self.quit(),
         }
@@ -369,7 +367,7 @@ impl Tui {
     /// Spawn a task to listen in the background for quit signals
     fn listen_for_signals(&self) {
         let messages_tx = self.messages_tx();
-        self.spawn(async move {
+        spawn_result(async move {
             signals().await?;
             messages_tx.send(Message::Quit);
             Ok(())
@@ -381,23 +379,21 @@ impl Tui {
     fn watch_collection(&self) -> anyhow::Result<impl Watcher> {
         // Spawn a watcher for the collection file
         let messages_tx = self.messages_tx();
-        let f = move |result: notify::Result<_>| {
-            match result {
-                // Only reload if the file *content* changes
-                Ok(
-                    event @ notify::Event {
-                        kind: notify::EventKind::Modify(ModifyKind::Data(_)),
-                        ..
-                    },
-                ) => {
-                    info!(?event, "Collection file changed, reloading");
-                    messages_tx.send(Message::CollectionStartReload);
-                }
-                // Do nothing for other event kinds
-                Ok(_) => {}
-                Err(err) => {
-                    error!(error = %err, "Error watching collection file");
-                }
+        let f = move |result: notify::Result<_>| match result {
+            // Only reload if the file *content* changes
+            Ok(
+                event @ notify::Event {
+                    kind: notify::EventKind::Modify(ModifyKind::Data(_)),
+                    ..
+                },
+            ) => {
+                info!(?event, "Collection file changed, reloading");
+                messages_tx.send(Message::CollectionStartReload);
+            }
+            // Do nothing for other event kinds
+            Ok(_) => {}
+            Err(err) => {
+                error!(error = %err, "Error watching collection file");
             }
         };
         let mut watcher = notify::recommended_watcher(f)?;
@@ -415,27 +411,34 @@ impl Tui {
         self.collection_file.collection = collection.into();
 
         // Rebuild the whole view, because tons of things can change
-        let database = self.database.clone();
-        let messages_tx = self.messages_tx();
-        let collection_file = &self.collection_file;
-        self.view = View::new(collection_file, database, messages_tx);
+        self.view = View::new(
+            &self.collection_file,
+            &self.request_store,
+            self.database.clone(),
+            self.messages_tx(),
+        );
+        self.view.notify(format!(
+            "Reloaded collection from {}",
+            self.collection_file.path().to_string_lossy()
+        ));
     }
 
     /// GOODBYE
     fn quit(&mut self) {
         info!("Initiating graceful shutdown");
         self.should_run = false;
+        // Kill all background tasks
+        CANCEL_TOKEN.cancel();
     }
 
     /// Draw the view onto the screen
     fn draw(&mut self) -> anyhow::Result<()> {
-        self.terminal
-            .draw(|frame| self.view.draw(frame, &self.request_store))?;
+        self.terminal.draw(|frame| self.view.draw(frame))?;
         Ok(())
     }
 
     /// Run a **blocking** subprocess that will take over the terminal. Used
-    /// for opening an external editor or viewer.
+    /// for opening an external editor or pager.
     fn run_command(&mut self, mut command: Command) -> anyhow::Result<()> {
         let span = info_span!("Running command", ?command).entered();
         let error_context = format!("Error spawning command `{command:?}`");
@@ -474,19 +477,17 @@ impl Tui {
     }
 
     /// Render URL for a request, then copy it to the clipboard
-    fn copy_request_url(
-        &self,
-        RequestConfig {
+    fn copy_request_url(&self) -> anyhow::Result<()> {
+        let RequestConfig {
             profile_id,
             recipe_id,
             options,
-        }: RequestConfig,
-    ) -> anyhow::Result<()> {
+        } = self.request_config()?;
         let seed = RequestSeed::new(recipe_id, options);
         let template_context = self.template_context(profile_id, false)?;
         let messages_tx = self.messages_tx();
         // Spawn a task to do the render+copy
-        self.spawn(async move {
+        spawn_result(async move {
             let url = TuiContext::get()
                 .http_engine
                 .build_url(seed, &template_context)
@@ -498,19 +499,17 @@ impl Tui {
     }
 
     /// Render body for a request, then copy it to the clipboard
-    fn copy_request_body(
-        &self,
-        RequestConfig {
+    fn copy_request_body(&self) -> anyhow::Result<()> {
+        let RequestConfig {
             profile_id,
             recipe_id,
             options,
-        }: RequestConfig,
-    ) -> anyhow::Result<()> {
+        } = self.request_config()?;
         let seed = RequestSeed::new(recipe_id, options);
         let template_context = self.template_context(profile_id, false)?;
         let messages_tx = self.messages_tx();
         // Spawn a task to do the render+copy
-        self.spawn(async move {
+        spawn_result(async move {
             let body = TuiContext::get()
                 .http_engine
                 .build_body(seed, &template_context)
@@ -526,19 +525,17 @@ impl Tui {
     }
 
     /// Render a request, then copy the equivalent curl command to the clipboard
-    fn copy_request_curl(
-        &self,
-        RequestConfig {
+    fn copy_request_curl(&self) -> anyhow::Result<()> {
+        let RequestConfig {
             profile_id,
             recipe_id,
             options,
-        }: RequestConfig,
-    ) -> anyhow::Result<()> {
+        } = self.request_config()?;
         let seed = RequestSeed::new(recipe_id, options);
         let template_context = self.template_context(profile_id, false)?;
         let messages_tx = self.messages_tx();
         // Spawn a task to do the render+copy
-        self.spawn(async move {
+        spawn_result(async move {
             let ticket = TuiContext::get()
                 .http_engine
                 .build(seed, &template_context)
@@ -574,19 +571,28 @@ impl Tui {
             // never parsed. This clone is cheap so we're being efficient!
             exchange.response.body.bytes().clone()
         });
-        self.spawn(save_file(self.messages_tx(), default_path, data));
+        spawn_result(save_file(self.messages_tx(), default_path, data));
         Ok(())
     }
 
+    /// Get the current request config for the selected recipe. The config
+    /// defines how to build a request. If no recipe is selected, this returns
+    /// an error. This should only be called in contexts where we can safely
+    /// assume that a recipe is selected (e.g. triggered via an action on a
+    /// recipe), so an error indicates a bug.
+    fn request_config(&self) -> anyhow::Result<RequestConfig> {
+        self.view
+            .request_config()
+            .ok_or_else(|| anyhow!("No recipe selected"))
+    }
+
     /// Launch an HTTP request in a separate task
-    fn send_request(
-        &mut self,
-        RequestConfig {
+    fn send_request(&mut self) -> anyhow::Result<()> {
+        let RequestConfig {
             profile_id,
             recipe_id,
             options,
-        }: RequestConfig,
-    ) -> anyhow::Result<()> {
+        } = self.request_config()?;
         // Launch the request in a separate task so it doesn't block.
         // These clones are all cheap.
 
@@ -597,10 +603,10 @@ impl Tui {
         let seed = RequestSeed::new(recipe_id.clone(), options);
         let request_id = seed.id;
 
-        // We can't use self.spawn here because HTTP errors are handled
-        // differently from all other error types
         let database = self.database.clone();
-        let join_handle = tokio::spawn(async move {
+        // Don't use spawn_result here, because errors are handled specially for
+        // requests
+        let join_handle = spawn(async move {
             // Build the request
             let result = TuiContext::get()
                 .http_engine
@@ -648,32 +654,17 @@ impl Tui {
         &self,
         template: Template,
         profile_id: Option<ProfileId>,
-        on_complete: Box<
-            dyn 'static + Send + Sync + FnOnce(Vec<TemplateChunk>),
-        >,
+        on_complete: Callback<Vec<TemplateChunk>>,
     ) -> anyhow::Result<()> {
         let context = self.template_context(profile_id, true)?;
-        let messages_tx = self.messages_tx();
-        tokio::spawn(async move {
+        spawn(async move {
             // Render chunks, then write them to the output destination
             let chunks = template.render_chunks(&context).await;
             on_complete(chunks);
-            // Trigger a draw
-            messages_tx.send(Message::TemplatePreviewComplete);
         });
         Ok(())
     }
 
-    /// Helper for spawning a fallible task. Any error in the resolved future
-    /// will be shown to the user in a modal.
-    fn spawn(
-        &self,
-        future: impl Future<Output = anyhow::Result<()>> + Send + 'static,
-    ) {
-        let messages_tx = self.messages_tx();
-        tokio::spawn(async move { future.await.reported(&messages_tx) });
-    }
-
     /// Expose app state to the templater. Most of the data has to be cloned out
     /// to be passed across async boundaries. This is annoying but in reality
     /// it should be small data.
@@ -718,13 +709,14 @@ impl Drop for Tui {
 fn initialize_panic_handler() {
     let original_hook = std::panic::take_hook();
     std::panic::set_hook(Box::new(move |panic_info| {
-        restore_terminal().unwrap();
+        let _ = restore_terminal();
         original_hook(panic_info);
     }));
 }
 
 /// Set up terminal for TUI
 fn initialize_terminal() -> anyhow::Result<Term> {
+    initialize_panic_handler();
     crossterm::terminal::enable_raw_mode()?;
     let mut stdout = io::stdout();
     crossterm::execute!(stdout, EnterAlternateScreen, EnableMouseCapture)?;
