@@ -4,7 +4,7 @@ use crate::{
         common::scrollbar::Scrollbar,
         context::UpdateContext,
         draw::{Draw, DrawMetadata},
-        event::{Event, EventHandler, Update},
+        event::{Event, EventHandler, OptionEvent},
         state::{Identified, StateCell},
     },
 };
@@ -12,7 +12,6 @@ use ratatui::{
     buffer::Buffer,
     layout::{Layout, Rect},
     prelude::{Alignment, Constraint},
-    style::Style,
     text::{Line, StyledGrapheme, Text},
     widgets::{Paragraph, ScrollbarOrientation},
     Frame,
@@ -47,8 +46,6 @@ pub struct TextWindowProps<'a> {
     /// Text to render. We take a reference because this component tends to
     /// contain a lot of text, and we don't want to force a clone on render
     pub text: &'a Identified<Text<'a>>,
-    /// Extra text to render below the text window
-    pub footer: Option<Text<'a>>,
     pub margins: ScrollbarMargins,
 }
 
@@ -158,7 +155,7 @@ impl TextWindow {
             // have to map grapheme number -> byte offset and cache that,
             // because skipping bytes is O(1) instead of O(n)
             let graphemes = line
-                .styled_graphemes(Style::default())
+                .styled_graphemes(text.style)
                 .skip(self.offset_x.get())
                 .take(self.window_width.get());
             let mut x = 0;
@@ -176,11 +173,8 @@ impl TextWindow {
 }
 
 impl EventHandler for TextWindow {
-    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Update {
-        let Some(action) = event.action() else {
-            return Update::Propagate(event);
-        };
-        match action {
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event.opt().action(|action, propagate| match action {
             Action::Up | Action::ScrollUp => self.scroll_up(1),
             Action::Down | Action::ScrollDown => self.scroll_down(1),
             Action::ScrollLeft => self.scroll_left(1),
@@ -189,9 +183,8 @@ impl EventHandler for TextWindow {
             Action::PageDown => self.scroll_down(self.window_height.get()),
             Action::Home => self.scroll_to(0),
             Action::End => self.scroll_to(usize::MAX),
-            _ => return Update::Propagate(event),
-        }
-        Update::Consumed
+            _ => propagate.set(),
+        })
     }
 }
 
@@ -264,25 +257,6 @@ impl<'a> Draw<TextWindowProps<'a>> for TextWindow {
         // Draw the text content
         self.render_chars(props.text, frame.buffer_mut(), text_area);
 
-        // Render the footer just below the text. If the text has maxed out the
-        // possible area, this will render beyond that. A bit hacky but in
-        // practice it works
-        if let Some(footer) = props.footer {
-            frame.render_widget(
-                footer,
-                Rect {
-                    x: text_area.x,
-                    y: text_area.y
-                        + (cmp::min(
-                            text_state.height,
-                            self.window_height.get(),
-                        )) as u16,
-                    width: text_area.width,
-                    height: 1,
-                },
-            );
-        }
-
         // Scrollbars
         if has_vertical_scroll {
             frame.render_widget(
@@ -331,20 +305,17 @@ mod tests {
         let text =
             Text::from("line 1\nline 2 is longer\nline 3\nline 4\nline 5")
                 .into();
-        let mut component = TestComponent::new(
-            &harness,
-            &terminal,
-            TextWindow::default(),
-            TextWindowProps {
-                text: &text,
-                // Don't overflow the frame
-                margins: ScrollbarMargins {
-                    right: 0,
-                    bottom: 0,
-                },
-                footer: None,
-            },
-        );
+        let mut component =
+            TestComponent::builder(&harness, &terminal, TextWindow::default())
+                .with_props(TextWindowProps {
+                    text: &text,
+                    // Don't overflow the frame
+                    margins: ScrollbarMargins {
+                        right: 0,
+                        bottom: 0,
+                    },
+                })
+                .build();
         terminal.assert_buffer_lines([
             vec![line_num(1), " line 1 ▲".into()],
             vec![line_num(2), " line 2 █".into()],
@@ -353,7 +324,7 @@ mod tests {
         ]);
 
         // Scroll down
-        component.send_key(KeyCode::Down).assert_empty();
+        component.int().send_key(KeyCode::Down).assert_empty();
         terminal.assert_buffer_lines([
             vec![line_num(2), " line 2 ▲".into()],
             vec![line_num(3), " line 3 █".into()],
@@ -362,8 +333,11 @@ mod tests {
         ]);
 
         // Scroll back up
-        component.send_key(KeyCode::Up).assert_empty();
-        component.send_key(KeyCode::Up).assert_empty(); // Does nothing
+        component
+            .int()
+            // Second does nothing
+            .send_keys([KeyCode::Up, KeyCode::Up])
+            .assert_empty();
         terminal.assert_buffer_lines([
             vec![line_num(1), " line 1 ▲".into()],
             vec![line_num(2), " line 2 █".into()],
@@ -373,12 +347,9 @@ mod tests {
 
         // Scroll right
         component
+            .int()
             .send_key_modifiers(KeyCode::Right, KeyModifiers::SHIFT)
-            .assert_empty();
-        component
             .send_key_modifiers(KeyCode::Right, KeyModifiers::SHIFT)
-            .assert_empty();
-        component
             .send_key_modifiers(KeyCode::Right, KeyModifiers::SHIFT)
             .assert_empty();
         terminal.assert_buffer_lines([
@@ -390,17 +361,13 @@ mod tests {
 
         // Scroll back left
         component
+            .int()
+            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
+            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
+            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
+            // Does nothing
             .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
             .assert_empty();
-        component
-            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
-            .assert_empty();
-        component
-            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
-            .assert_empty();
-        component
-            .send_key_modifiers(KeyCode::Left, KeyModifiers::SHIFT)
-            .assert_empty(); // Does nothing
         terminal.assert_buffer_lines([
             vec![line_num(1), " line 1 ▲".into()],
             vec![line_num(2), " line 2 █".into()],
@@ -416,20 +383,16 @@ mod tests {
     ) {
         let text =
             Text::from("intro\n💚💙💜 this is a longer line\noutro").into();
-        TestComponent::new(
-            &harness,
-            &terminal,
-            TextWindow::default(),
-            TextWindowProps {
+        TestComponent::builder(&harness, &terminal, TextWindow::default())
+            .with_props(TextWindowProps {
                 text: &text,
                 // Don't overflow the frame
                 margins: ScrollbarMargins {
                     right: 0,
                     bottom: 0,
                 },
-                footer: None,
-            },
-        );
+            })
+            .build();
         terminal.assert_buffer_lines([
             vec![line_num(1), " intro                            ".into()],
             vec![line_num(2), " 💚💙💜 this is a longer line    ".into()],
@@ -443,20 +406,16 @@ mod tests {
         harness: TestHarness,
     ) {
         let text = Text::raw("💚💙💜💚💙💜").into();
-        TestComponent::new(
-            &harness,
-            &terminal,
-            TextWindow::default(),
-            TextWindowProps {
+        TestComponent::builder(&harness, &terminal, TextWindow::default())
+            .with_props(TextWindowProps {
                 text: &text,
                 // Don't overflow the frame
                 margins: ScrollbarMargins {
                     right: 0,
                     bottom: 0,
                 },
-                footer: None,
-            },
-        );
+            })
+            .build();
         terminal.assert_buffer_lines([
             vec![line_num(1), " 💚💙💜💚".into()],
             vec![line_num(0), " ◀■■■■══▶".into()],
@@ -473,20 +432,17 @@ mod tests {
         let text =
             Text::from_iter(["1 this is a long line", "2", "3", "4", "5"])
                 .into();
-        let mut component = TestComponent::new(
-            &harness,
-            &terminal,
-            TextWindow::default(),
-            TextWindowProps {
-                text: &text,
-                // Don't overflow the frame
-                margins: ScrollbarMargins {
-                    right: 0,
-                    bottom: 0,
-                },
-                footer: None,
-            },
-        );
+        let mut component =
+            TestComponent::builder(&harness, &terminal, TextWindow::default())
+                .with_props(TextWindowProps {
+                    text: &text,
+                    // Don't overflow the frame
+                    margins: ScrollbarMargins {
+                        right: 0,
+                        bottom: 0,
+                    },
+                })
+                .build();
 
         // Scroll out a bit
         component.data_mut().scroll_down(2);
@@ -501,9 +457,8 @@ mod tests {
                 right: 0,
                 bottom: 0,
             },
-            footer: None,
         });
-        component.drain_draw().assert_empty();
+        component.int().drain_draw().assert_empty();
 
         assert_eq!(component.data().offset_x.get(), 8);
         assert_eq!(component.data().offset_y.get(), 1);
@@ -516,23 +471,20 @@ mod tests {
         let text =
             Text::from_iter(["1 this is a long line", "2", "3", "4", "5"])
                 .into();
-        let mut component = TestComponent::new(
-            &harness,
-            &terminal,
-            TextWindow::default(),
-            TextWindowProps {
-                text: &text,
-                // Don't overflow the frame
-                margins: ScrollbarMargins {
-                    right: 0,
-                    bottom: 0,
-                },
-                footer: None,
-            },
-        );
+        let mut component =
+            TestComponent::builder(&harness, &terminal, TextWindow::default())
+                .with_props(TextWindowProps {
+                    text: &text,
+                    // Don't overflow the frame
+                    margins: ScrollbarMargins {
+                        right: 0,
+                        bottom: 0,
+                    },
+                })
+                .build();
 
         component.set_area(Rect::new(0, 0, 10, 3));
-        component.drain_draw().assert_empty();
+        component.int().drain_draw().assert_empty();
 
         // Scroll out a bit
         component.data_mut().scroll_down(2);
@@ -541,7 +493,7 @@ mod tests {
         assert_eq!(component.data().offset_y.get(), 2);
 
         component.set_area(Rect::new(0, 0, 15, 4));
-        component.drain_draw().assert_empty();
+        component.int().drain_draw().assert_empty();
 
         assert_eq!(component.data().offset_x.get(), 8);
         assert_eq!(component.data().offset_y.get(), 1);
