@@ -3,9 +3,10 @@
 //! component state.
 
 use crate::view::{
+    common::actions::MenuAction,
     context::UpdateContext,
     draw::{Draw, DrawMetadata},
-    event::{Child, Event, ToChild, Update},
+    event::{Child, Emitter, Event, LocalEvent, ToChild, ToEmitter},
 };
 use crossterm::event::MouseEvent;
 use derive_more::Display;
@@ -75,6 +76,31 @@ impl<T> Component<T> {
         self.name
     }
 
+    /// Collect all available menu actions from all **focused** components. This
+    /// takes a mutable reference so we don't have to duplicate the code that
+    /// provides children; it will *not* mutate anything.
+    pub fn collect_actions(&mut self) -> Vec<MenuAction>
+    where
+        T: ToChild,
+    {
+        fn inner(
+            actions: &mut Vec<MenuAction>,
+            mut component: Component<Child>,
+        ) {
+            // Only include actions from visible+focused components
+            if component.is_visible() && component.has_focus() {
+                actions.extend(component.data().menu_actions());
+                for child in component.data_mut().children() {
+                    inner(actions, child)
+                }
+            }
+        }
+
+        let mut actions = Vec::new();
+        inner(&mut actions, self.to_child_mut());
+        actions
+    }
+
     /// Handle an event for this component *or* its children, starting at the
     /// lowest descendant. Recursively walk up the tree until a component
     /// consumes the event.
@@ -83,13 +109,13 @@ impl<T> Component<T> {
         &mut self,
         context: &mut UpdateContext,
         mut event: Event,
-    ) -> Update
+    ) -> Option<Event>
     where
         T: ToChild,
     {
         // If we can't handle the event, our children can't either
         if !self.should_handle(&event) {
-            return Update::Propagate(event);
+            return Some(event);
         }
 
         let mut self_dyn = self.data_mut().to_child_mut();
@@ -99,13 +125,13 @@ impl<T> Component<T> {
             // RECURSION
             let update = child.update_all(context, event);
             match update {
-                Update::Propagate(returned) => {
+                Some(returned) => {
                     // Keep going to the next child. The propagated event
                     // *should* just be whatever we passed in, but we have
                     // no way of verifying that
                     event = returned;
                 }
-                Update::Consumed => {
+                None => {
                     return update;
                 }
             }
@@ -113,7 +139,7 @@ impl<T> Component<T> {
 
         // None of our children handled it, we'll take it ourselves. Event is
         // already traced in the root span, so don't dupe it.
-        (trace_span!("component.update")).in_scope(|| {
+        trace_span!("component.update").in_scope(|| {
             let update = self_dyn.update(context, event);
             trace!(?update);
             update
@@ -137,7 +163,7 @@ impl<T> Component<T> {
         use crossterm::event::Event::*;
         if let Event::Input { event, .. } = event {
             match event {
-                Key(_) | Paste(_) => self.metadata.get().has_focus(),
+                Key(_) | Paste(_) => self.has_focus(),
 
                 Mouse(mouse_event) => {
                     // Check if the mouse is over the component
@@ -160,6 +186,11 @@ impl<T> Component<T> {
         VISIBLE_COMPONENTS.with_borrow(|tree| tree.contains(&self.id))
     }
 
+    /// Was this component in focus during the previous draw phase?
+    fn has_focus(&self) -> bool {
+        self.metadata.get().has_focus()
+    }
+
     /// Did the given mouse event occur over/on this component?
     fn intersects(&self, mouse_event: &MouseEvent) -> bool {
         self.is_visible()
@@ -267,6 +298,16 @@ impl<T> From<T> for Component<T> {
     }
 }
 
+impl<E, T> ToEmitter<E> for Component<T>
+where
+    E: LocalEvent,
+    T: ToEmitter<E>,
+{
+    fn to_emitter(&self) -> Emitter<E> {
+        self.data().to_emitter()
+    }
+}
+
 /// Unique ID to refer to a single component. Generally components are persisent
 /// throughout the lifespan of the view so this will be too, but it's only
 /// *necessary* for it to be consistent from one draw phase to the next event
@@ -347,7 +388,7 @@ mod tests {
     use super::*;
     use crate::{
         test_util::{harness, terminal, TestHarness, TestTerminal},
-        view::event::{EventHandler, Update},
+        view::event::EventHandler,
     };
     use crossterm::event::{
         KeyCode, KeyEvent, KeyEventKind, KeyEventState, KeyModifiers,
@@ -389,9 +430,9 @@ mod tests {
     }
 
     impl EventHandler for Branch {
-        fn update(&mut self, _: &mut UpdateContext, _: Event) -> Update {
+        fn update(&mut self, _: &mut UpdateContext, _: Event) -> Option<Event> {
             self.count += 1;
-            Update::Consumed
+            None
         }
 
         fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -443,9 +484,9 @@ mod tests {
     }
 
     impl EventHandler for Leaf {
-        fn update(&mut self, _: &mut UpdateContext, _: Event) -> Update {
+        fn update(&mut self, _: &mut UpdateContext, _: Event) -> Option<Event> {
             self.count += 1;
-            Update::Consumed
+            None
         }
     }
 
@@ -630,7 +671,7 @@ mod tests {
                 },
                 keyboard_event()
             ),
-            Update::Propagate(_)
+            Some(_)
         );
     }
 
@@ -665,7 +706,7 @@ mod tests {
                 },
                 keyboard_event()
             ),
-            Update::Propagate(_)
+            Some(_)
         );
     }
 }
