@@ -6,25 +6,31 @@ use crate::view::{
         button::ButtonGroup,
         list::List,
         modal::{IntoModal, Modal},
-        text_box::TextBox,
+        text_box::{TextBox, TextBoxEvent, TextBoxProps},
     },
     component::Component,
     context::UpdateContext,
     draw::{Draw, DrawMetadata, Generate},
-    event::{Child, Event, EventHandler, Update},
-    state::{select::SelectState, Notification},
-    Confirm, ModalPriority, ViewContext,
+    event::{Child, Event, EventHandler, OptionEvent, ToEmitter},
+    state::{
+        select::{SelectState, SelectStateEvent, SelectStateEventType},
+        Notification,
+    },
+    Confirm, ModalPriority,
 };
 use derive_more::Display;
 use ratatui::{
     prelude::Constraint,
     text::{Line, Text},
-    widgets::{Paragraph, Wrap},
+    widgets::Paragraph,
     Frame,
 };
 use slumber_core::template::{Prompt, Select};
+use std::fmt::Debug;
 use strum::{EnumCount, EnumIter};
 
+/// Modal to display an error. Typically the error is [anyhow::Error], but it
+/// could also be wrapped in a smart pointer.
 #[derive(Debug)]
 pub struct ErrorModal(anyhow::Error);
 
@@ -46,10 +52,7 @@ impl EventHandler for ErrorModal {}
 
 impl Draw for ErrorModal {
     fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
-        frame.render_widget(
-            Paragraph::new(self.0.generate()).wrap(Wrap::default()),
-            metadata.area(),
-        );
+        frame.render_widget(self.0.generate(), metadata.area());
     }
 }
 
@@ -83,20 +86,9 @@ impl TextBoxModal {
         text_box: TextBox,
         on_submit: impl 'static + FnOnce(String),
     ) -> Self {
-        let text_box = text_box
-            // Make sure cancel gets propagated to close the modal
-            .on_cancel(|| {
-                ViewContext::push_event(Event::CloseModal { submitted: false })
-            })
-            .on_submit(move || {
-                // We have to defer submission to on_close, because we need the
-                // owned value of `self.prompt`
-                ViewContext::push_event(Event::CloseModal { submitted: true });
-            })
-            .into();
         Self {
             title,
-            text_box,
+            text_box: text_box.into(),
             on_submit: Box::new(on_submit),
         }
     }
@@ -120,6 +112,23 @@ impl Modal for TextBoxModal {
 }
 
 impl EventHandler for TextBoxModal {
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event
+            .opt()
+            .emitted(self.text_box.to_emitter(), |event| match event {
+                TextBoxEvent::Focus | TextBoxEvent::Change => {}
+                TextBoxEvent::Cancel => {
+                    // Propagate cancel to close the modal
+                    self.close(false);
+                }
+                TextBoxEvent::Submit => {
+                    // We have to defer submission to on_close, because we need
+                    // the owned value of `self.on_submit`
+                    self.close(true);
+                }
+            })
+    }
+
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
         vec![self.text_box.to_child_mut()]
     }
@@ -127,7 +136,12 @@ impl EventHandler for TextBoxModal {
 
 impl Draw for TextBoxModal {
     fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
-        self.text_box.draw(frame, (), metadata.area(), true);
+        self.text_box.draw(
+            frame,
+            TextBoxProps::default(),
+            metadata.area(),
+            true,
+        );
     }
 }
 
@@ -168,11 +182,7 @@ impl SelectListModal {
         Self {
             title,
             options: SelectState::builder(options)
-                .on_submit(move |_| {
-                    ViewContext::push_event(Event::CloseModal {
-                        submitted: true,
-                    });
-                })
+                .subscribe([SelectStateEventType::Submit])
                 .build()
                 .into(),
             on_submit: Box::new(on_submit),
@@ -204,12 +214,25 @@ impl Modal for SelectListModal {
         // because the user selected a value (submitted).
         if submitted {
             // Return the user's value and close the prompt
-            (self.on_submit)(self.options.data().selected().unwrap().clone());
+            (self.on_submit)(
+                self.options
+                    .into_data()
+                    .into_selected()
+                    .expect("User submitted something"),
+            );
         }
     }
 }
 
 impl EventHandler for SelectListModal {
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event.opt().emitted(self.options.to_emitter(), |event| {
+            if let SelectStateEvent::Submit(_) = event {
+                self.close(true);
+            }
+        })
+    }
+
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
         vec![self.options.to_child_mut()]
     }
@@ -299,15 +322,12 @@ impl Modal for ConfirmModal {
 }
 
 impl EventHandler for ConfirmModal {
-    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Update {
-        // When user selects a button, send the response and close
-        let Some(button) = event.local::<ConfirmButton>() else {
-            return Update::Propagate(event);
-        };
-
-        self.answer = *button == ConfirmButton::Yes;
-        ViewContext::push_event(Event::CloseModal { submitted: true });
-        Update::Consumed
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event.opt().emitted(self.buttons.to_emitter(), |button| {
+            // When user selects a button, send the response and close
+            self.answer = button == ConfirmButton::Yes;
+            self.close(true);
+        })
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
