@@ -2,25 +2,28 @@
 
 use crate::{
     http::RequestState,
-    message::Message,
+    message::{Message, RequestConfig},
     util::ResultReported,
     view::{
-        common::actions::ActionsModal,
+        common::{
+            actions::{IntoMenuAction, MenuAction},
+            modal::Modal,
+        },
         component::{
-            exchange_pane::{ExchangePane, ExchangePaneProps},
+            exchange_pane::{ExchangePane, ExchangePaneEvent},
             help::HelpModal,
             profile_select::ProfilePane,
-            recipe_list::RecipeListPane,
-            recipe_pane::{RecipeMenuAction, RecipePane, RecipePaneProps},
+            recipe_list::{RecipeListPane, RecipeListPaneEvent},
+            recipe_pane::{RecipePane, RecipePaneEvent, RecipePaneProps},
         },
         context::UpdateContext,
-        draw::{Draw, DrawMetadata, ToStringGenerate},
-        event::{Child, Event, EventHandler, Update},
-        state::fixed_select::FixedSelectState,
-        util::{
-            persistence::{Persisted, PersistedLazy},
-            view_text,
+        draw::{Draw, DrawMetadata},
+        event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
+        state::{
+            fixed_select::FixedSelectState,
+            select::{SelectStateEvent, SelectStateEventType},
         },
+        util::persistence::{Persisted, PersistedLazy},
         Component, ViewContext,
     },
 };
@@ -34,9 +37,9 @@ use ratatui::{
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
 use slumber_core::collection::{
-    Collection, ProfileId, RecipeId, RecipeNodeDiscriminants,
+    Collection, ProfileId, RecipeId, RecipeNodeType,
 };
-use strum::{EnumCount, EnumIter};
+use strum::{EnumCount, EnumIter, IntoEnumIterator};
 
 /// Primary TUI view, which shows request/response panes
 #[derive(Debug)]
@@ -51,86 +54,39 @@ pub struct PrimaryView {
     recipe_list_pane: Component<RecipeListPane>,
     recipe_pane: Component<RecipePane>,
     exchange_pane: Component<ExchangePane>,
+
+    global_actions_emitter: Emitter<GlobalMenuAction>,
 }
 
-#[cfg_attr(test, derive(Clone))]
-pub struct PrimaryViewProps<'a> {
-    pub selected_request: Option<&'a RequestState>,
-}
-
-/// Selectable panes in the primary view mode
-#[derive(
-    Copy,
-    Clone,
-    Debug,
-    Default,
-    Display,
-    EnumCount,
-    EnumIter,
-    PartialEq,
-    Serialize,
-    Deserialize,
-)]
-pub enum PrimaryPane {
-    #[default]
-    RecipeList,
-    Recipe,
-    Exchange,
-}
-
-/// Panes that can be fullscreened. This is separate from [PrimaryPane] because
-/// it makes it easy to check when we should exit fullscreen mode.
-#[derive(Copy, Clone, Debug, PartialEq, Serialize, Deserialize)]
-enum FullscreenMode {
-    /// Fullscreen the active request recipe
-    Recipe,
-    /// Fullscreen the active request/response exchange
-    Exchange,
-}
-
-/// Persistence key for fullscreen mode
-#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
-#[persisted(Option<FullscreenMode>)]
-struct FullscreenModeKey;
-
-/// Event triggered when selected pane changes, so we can exit fullscreen
-#[derive(Debug)]
-struct PaneChanged;
-
-/// Action menu items. This is the fallback menu if none of our children have
-/// one
-#[derive(
-    Copy, Clone, Debug, Default, Display, EnumCount, EnumIter, PartialEq,
-)]
-enum MenuAction {
-    #[default]
-    #[display("Edit Collection")]
-    EditCollection,
-}
-impl ToStringGenerate for MenuAction {}
-
 impl PrimaryView {
-    pub fn new(collection: &Collection) -> Self {
-        let profile_pane = ProfilePane::new(collection).into();
-        let recipe_list_pane = RecipeListPane::new(&collection.recipes).into();
-        let selected_pane = FixedSelectState::builder()
-            // Changing panes kicks us out of fullscreen
-            .on_select(|_| {
-                ViewContext::push_event(Event::new_local(PaneChanged))
-            })
-            .build();
+    pub fn new(
+        collection: &Collection,
+        selected_request: Option<&RequestState>,
+    ) -> Self {
+        let profile_pane = ProfilePane::new(collection);
+        let recipe_list_pane = RecipeListPane::new(&collection.recipes);
+        let exchange_pane = ExchangePane::new(
+            selected_request,
+            recipe_list_pane
+                .selected_node()
+                .map(|(_, node_type)| node_type),
+        );
 
         Self {
             selected_pane: PersistedLazy::new(
                 SingletonKey::default(),
-                selected_pane,
+                FixedSelectState::builder()
+                    .subscribe([SelectStateEventType::Select])
+                    .build(),
             ),
-            fullscreen_mode: Persisted::default(),
+            fullscreen_mode: Default::default(),
 
-            recipe_list_pane,
-            profile_pane,
+            recipe_list_pane: recipe_list_pane.into(),
+            profile_pane: profile_pane.into(),
             recipe_pane: Default::default(),
-            exchange_pane: Default::default(),
+            exchange_pane: exchange_pane.into(),
+
+            global_actions_emitter: Default::default(),
         }
     }
 
@@ -141,7 +97,7 @@ impl PrimaryView {
             .data()
             .selected_node()
             .and_then(|(id, kind)| {
-                if matches!(kind, RecipeNodeDiscriminants::Recipe) {
+                if matches!(kind, RecipeNodeType::Recipe) {
                     Some(id)
                 } else {
                     None
@@ -154,65 +110,26 @@ impl PrimaryView {
         self.profile_pane.data().selected_profile_id()
     }
 
-    /// Draw the "normal" view, when nothing is fullscreened
-    fn draw_all_panes(
-        &self,
-        frame: &mut Frame,
-        props: PrimaryViewProps,
-        area: Rect,
+    /// Set the state of the currently selected request. Call whenever a new
+    /// request is selected, or the selected request changes state
+    pub fn set_request_state(
+        &mut self,
+        selected_request: Option<&RequestState>,
     ) {
-        // Split the main pane horizontally
-        let [left_area, right_area] =
-            Layout::horizontal([Constraint::Max(40), Constraint::Min(40)])
-                .areas(area);
+        self.exchange_pane = ExchangePane::new(
+            selected_request,
+            self.recipe_list_pane
+                .data()
+                .selected_node()
+                .map(|(_, node_type)| node_type),
+        )
+        .into();
+    }
 
-        let [profile_area, recipes_area] =
-            Layout::vertical([Constraint::Length(3), Constraint::Min(0)])
-                .areas(left_area);
-        let [recipe_area, request_response_area] =
-            self.get_right_column_layout(right_area);
-
-        self.profile_pane.draw(frame, (), profile_area, true);
-        self.recipe_list_pane.draw(
-            frame,
-            (),
-            recipes_area,
-            self.is_selected(PrimaryPane::RecipeList),
-        );
-
-        let (selected_recipe_id, selected_recipe_kind) =
-            match self.recipe_list_pane.data().selected_node() {
-                Some((selected_recipe_id, selected_recipe_kind)) => {
-                    (Some(selected_recipe_id), Some(selected_recipe_kind))
-                }
-                None => (None, None),
-            };
-        let collection = ViewContext::collection();
-        let selected_recipe_node = selected_recipe_id.and_then(|id| {
-            collection
-                .recipes
-                .try_get(id)
-                .reported(&ViewContext::messages_tx())
-        });
-        self.recipe_pane.draw(
-            frame,
-            RecipePaneProps {
-                selected_recipe_node,
-                selected_profile_id: self.selected_profile_id(),
-            },
-            recipe_area,
-            self.is_selected(PrimaryPane::Recipe),
-        );
-
-        self.exchange_pane.draw(
-            frame,
-            ExchangePaneProps {
-                selected_recipe_kind,
-                request_state: props.selected_request,
-            },
-            request_response_area,
-            self.is_selected(PrimaryPane::Exchange),
-        );
+    /// Get a definition of the request that should be sent from the current
+    /// recipe settings
+    pub fn request_config(&self) -> Option<RequestConfig> {
+        self.recipe_pane.data().request_config()
     }
 
     /// Is the given pane selected?
@@ -241,6 +158,58 @@ impl PrimaryView {
         }
     }
 
+    /// Get the current placement and focus for all panes, according to current
+    /// selection and fullscreen state. We always draw all panes so they can
+    /// perform their state updates. To hide them we just render to an empty
+    /// rect.
+    fn panes(&self, area: Rect) -> Panes {
+        if let Some(fullscreen_mode) = *self.fullscreen_mode {
+            match fullscreen_mode {
+                FullscreenMode::Recipe => Panes {
+                    profile: PaneState::default(),
+                    recipe_list: PaneState::default(),
+                    recipe: PaneState { area, focus: true },
+                    exchange: PaneState::default(),
+                },
+                FullscreenMode::Exchange => Panes {
+                    profile: PaneState::default(),
+                    recipe_list: PaneState::default(),
+                    recipe: PaneState::default(),
+                    exchange: PaneState { area, focus: true },
+                },
+            }
+        } else {
+            // Split the main pane horizontally
+            let [left_area, right_area] =
+                Layout::horizontal([Constraint::Max(40), Constraint::Min(40)])
+                    .areas(area);
+
+            let [profile_area, recipe_list_area] =
+                Layout::vertical([Constraint::Length(3), Constraint::Min(0)])
+                    .areas(left_area);
+            let [recipe_area, exchange_area] =
+                self.get_right_column_layout(right_area);
+            Panes {
+                profile: PaneState {
+                    area: profile_area,
+                    focus: true,
+                },
+                recipe_list: PaneState {
+                    area: recipe_list_area,
+                    focus: self.is_selected(PrimaryPane::RecipeList),
+                },
+                recipe: PaneState {
+                    area: recipe_area,
+                    focus: self.is_selected(PrimaryPane::Recipe),
+                },
+                exchange: PaneState {
+                    area: exchange_area,
+                    focus: self.is_selected(PrimaryPane::Exchange),
+                },
+            }
+        }
+    }
+
     /// Get layout for the right column of panes
     fn get_right_column_layout(&self, area: Rect) -> [Rect; 2] {
         // Split right column vertically. Expand the currently selected pane
@@ -256,67 +225,26 @@ impl PrimaryView {
         .areas(area)
     }
 
-    /// Handle menu actions for recipe list or detail panes. We handle this here
-    /// for code de-duplication, and because we have access to all the needed
-    /// context.
-    fn handle_recipe_menu_action(&self, action: RecipeMenuAction) {
-        // If no recipes are available, we can't do anything
-        let Some(config) = self.recipe_pane.data().request_config() else {
-            return;
-        };
-
-        match action {
-            RecipeMenuAction::EditCollection => {
-                ViewContext::send_message(Message::CollectionEdit)
-            }
-            RecipeMenuAction::CopyUrl => {
-                ViewContext::send_message(Message::CopyRequestUrl(config))
-            }
-            RecipeMenuAction::CopyCurl => {
-                ViewContext::send_message(Message::CopyRequestCurl(config))
-            }
-            RecipeMenuAction::CopyBody => {
-                ViewContext::send_message(Message::CopyRequestBody(config))
-            }
-            RecipeMenuAction::ViewBody => {
-                self.recipe_pane.data().with_body_text(view_text)
-            }
-        }
+    /// Send a request for the currently selected recipe
+    fn send_request(&self) {
+        ViewContext::send_message(Message::HttpBeginRequest);
     }
 }
 
 impl EventHandler for PrimaryView {
-    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Update {
-        match &event {
-            // Input messages
-            Event::Input {
-                action: Some(action),
-                event: _,
-            } => match action {
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event
+            .opt()
+            .action(|action, propagate| match action {
                 Action::PreviousPane => self.selected_pane.get_mut().previous(),
                 Action::NextPane => self.selected_pane.get_mut().next(),
-                Action::Submit => {
-                    // Send a request from anywhere
-                    if let Some(config) =
-                        self.recipe_pane.data().request_config()
-                    {
-                        ViewContext::send_message(Message::HttpBeginRequest(
-                            config,
-                        ));
-                    }
-                }
-                Action::OpenActions => {
-                    ViewContext::open_modal::<ActionsModal<MenuAction>>(
-                        Default::default(),
-                    );
-                }
-                Action::OpenHelp => {
-                    ViewContext::open_modal::<HelpModal>(Default::default());
-                }
+                // Send a request from anywhere
+                Action::Submit => self.send_request(),
+                Action::OpenHelp => HelpModal.open(),
 
                 // Pane hotkeys
                 Action::SelectProfileList => {
-                    self.profile_pane.data().open_modal()
+                    self.profile_pane.data_mut().open_modal()
                 }
                 Action::SelectRecipeList => self
                     .selected_pane
@@ -347,34 +275,45 @@ impl EventHandler for PrimaryView {
                 Action::Cancel if self.fullscreen_mode.is_some() => {
                     *self.fullscreen_mode.get_mut() = None;
                 }
-                _ => return Update::Propagate(event),
-            },
-
-            Event::Local(local) => {
-                if let Some(PaneChanged) = local.downcast_ref() {
+                _ => propagate.set(),
+            })
+            .emitted(self.selected_pane.to_emitter(), |event| {
+                if let SelectStateEvent::Select(_) = event {
+                    // Exit fullscreen when pane changes
                     self.maybe_exit_fullscreen();
-                } else if let Some(pane) = local.downcast_ref::<PrimaryPane>() {
-                    // Children can select themselves by sending PrimaryPane
-                    self.selected_pane.get_mut().select(pane);
-                } else if let Some(action) =
-                    local.downcast_ref::<RecipeMenuAction>()
-                {
-                    self.handle_recipe_menu_action(*action);
-                } else if let Some(action) = local.downcast_ref::<MenuAction>()
-                {
-                    match action {
-                        MenuAction::EditCollection => {
-                            ViewContext::send_message(Message::CollectionEdit)
-                        }
-                    }
-                } else {
-                    return Update::Propagate(event);
                 }
-            }
+            })
+            .emitted(self.recipe_list_pane.to_emitter(), |event| match event {
+                RecipeListPaneEvent::Click => {
+                    self.selected_pane
+                        .get_mut()
+                        .select(&PrimaryPane::RecipeList);
+                }
+            })
+            .emitted(self.recipe_pane.to_emitter(), |event| match event {
+                RecipePaneEvent::Click => {
+                    self.selected_pane.get_mut().select(&PrimaryPane::Recipe);
+                }
+            })
+            .emitted(self.exchange_pane.to_emitter(), |event| match event {
+                ExchangePaneEvent::Click => {
+                    self.selected_pane.get_mut().select(&PrimaryPane::Exchange)
+                }
+            })
+            .emitted(self.global_actions_emitter, |menu_action| {
+                // Handle our own menu action type
+                match menu_action {
+                    GlobalMenuAction::EditCollection => {
+                        ViewContext::send_message(Message::CollectionEdit)
+                    }
+                }
+            })
+    }
 
-            _ => return Update::Propagate(event),
-        }
-        Update::Consumed
+    fn menu_actions(&self) -> Vec<MenuAction> {
+        GlobalMenuAction::iter()
+            .map(MenuAction::with_data(self, self.global_actions_emitter))
+            .collect()
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -387,53 +326,115 @@ impl EventHandler for PrimaryView {
     }
 }
 
-impl<'a> Draw<PrimaryViewProps<'a>> for PrimaryView {
-    fn draw(
-        &self,
-        frame: &mut Frame,
-        props: PrimaryViewProps<'a>,
-        metadata: DrawMetadata,
-    ) {
-        match *self.fullscreen_mode {
-            None => self.draw_all_panes(frame, props, metadata.area()),
-            Some(FullscreenMode::Recipe) => {
-                let collection = ViewContext::collection();
-                let selected_recipe_node =
-                    self.recipe_list_pane.data().selected_node().and_then(
-                        |(id, _)| {
-                            collection
-                                .recipes
-                                .try_get(id)
-                                .reported(&ViewContext::messages_tx())
-                        },
-                    );
-                self.recipe_pane.draw(
-                    frame,
-                    RecipePaneProps {
-                        selected_recipe_node,
-                        selected_profile_id: self.selected_profile_id(),
-                    },
-                    metadata.area(),
-                    self.is_selected(PrimaryPane::Recipe),
-                );
-            }
-            Some(FullscreenMode::Exchange) => self.exchange_pane.draw(
-                frame,
-                ExchangePaneProps {
-                    selected_recipe_kind: self
-                        .recipe_list_pane
-                        .data()
-                        .selected_node()
-                        .map(|(_, kind)| kind),
-                    request_state: props.selected_request,
-                },
-                metadata.area(),
-                true,
-            ),
-        }
+impl Draw for PrimaryView {
+    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+        // We draw all panes regardless of fullscreen state, so they can run
+        // their necessary state updates. We just give the hidden panes an empty
+        // rect to draw into so they don't appear at all
+        let panes = self.panes(metadata.area());
+
+        self.profile_pane.draw(
+            frame,
+            (),
+            panes.profile.area,
+            panes.profile.focus,
+        );
+        self.recipe_list_pane.draw(
+            frame,
+            (),
+            panes.recipe_list.area,
+            panes.recipe_list.focus,
+        );
+
+        let collection = ViewContext::collection();
+        let selected_recipe_node = self
+            .recipe_list_pane
+            .data()
+            .selected_node()
+            .and_then(|(id, _)| {
+                collection
+                    .recipes
+                    .try_get(id)
+                    .reported(&ViewContext::messages_tx())
+            });
+        self.recipe_pane.draw(
+            frame,
+            RecipePaneProps {
+                selected_recipe_node,
+                selected_profile_id: self.selected_profile_id(),
+            },
+            panes.recipe.area,
+            panes.recipe.focus,
+        );
+
+        self.exchange_pane.draw(
+            frame,
+            (),
+            panes.exchange.area,
+            panes.exchange.focus,
+        );
     }
 }
 
+/// Selectable panes in the primary view mode
+#[derive(
+    Copy,
+    Clone,
+    Debug,
+    Default,
+    Display,
+    EnumCount,
+    EnumIter,
+    PartialEq,
+    Serialize,
+    Deserialize,
+)]
+enum PrimaryPane {
+    #[default]
+    RecipeList,
+    Recipe,
+    Exchange,
+}
+
+/// Panes that can be fullscreened. This is separate from [PrimaryPane] because
+/// it makes it easy to check when we should exit fullscreen mode.
+#[derive(Copy, Clone, Debug, PartialEq, Serialize, Deserialize)]
+enum FullscreenMode {
+    /// Fullscreen the active request recipe
+    Recipe,
+    /// Fullscreen the active request/response exchange
+    Exchange,
+}
+
+/// Persistence key for fullscreen mode
+#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
+#[persisted(Option<FullscreenMode>)]
+struct FullscreenModeKey;
+
+/// Menu actions available in all contexts
+#[derive(Copy, Clone, Debug, Display, EnumIter)]
+enum GlobalMenuAction {
+    #[display("Edit Collection")]
+    EditCollection,
+}
+
+impl IntoMenuAction<PrimaryView> for GlobalMenuAction {}
+
+/// Helper for adjusting pane behavior according to state
+struct Panes {
+    profile: PaneState,
+    recipe_list: PaneState,
+    recipe: PaneState,
+    exchange: PaneState,
+}
+
+/// Helper for adjusting pane behavior according to state
+#[derive(Default)]
+struct PaneState {
+    area: Rect,
+    focus: bool,
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -444,6 +445,7 @@ mod tests {
             test_util::TestComponent, util::persistence::DatabasePersistedStore,
         },
     };
+    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
     use slumber_core::{assert_matches, http::BuildOptions};
@@ -452,15 +454,13 @@ mod tests {
     fn create_component<'term>(
         harness: &mut TestHarness,
         terminal: &'term TestTerminal,
-    ) -> TestComponent<'term, PrimaryView, PrimaryViewProps<'static>> {
-        let view = PrimaryView::new(&harness.collection);
-        let component = TestComponent::new(
-            harness,
-            terminal,
-            view,
-            PrimaryViewProps {
-                selected_request: None,
-            },
+    ) -> TestComponent<'term, PrimaryView, ()> {
+        let view = PrimaryView::new(&harness.collection, None);
+        let mut component = TestComponent::new(harness, terminal, view);
+        // Initial events
+        assert_matches!(
+            component.int().drain_draw().events(),
+            &[Event::HttpSelectRequest(None)]
         );
         // Clear template preview messages so we can test what we want
         harness.clear_messages();
@@ -490,66 +490,66 @@ mod tests {
         );
     }
 
+    /// Test the request_config() getter
+    #[rstest]
+    fn test_request_config(mut harness: TestHarness, terminal: TestTerminal) {
+        let component = create_component(&mut harness, &terminal);
+        let expected_config = RequestConfig {
+            recipe_id: harness.collection.first_recipe_id().clone(),
+            profile_id: Some(harness.collection.first_profile_id().clone()),
+            options: BuildOptions::default(),
+        };
+        assert_eq!(component.data().request_config(), Some(expected_config));
+    }
+
+    /// Test "Edit Collection" action
+    #[rstest]
+    fn test_edit_collection(mut harness: TestHarness, terminal: TestTerminal) {
+        let mut component = create_component(&mut harness, &terminal);
+        component.int().drain_draw().assert_empty();
+
+        harness.clear_messages(); // Clear init junk
+
+        component
+            .int()
+            .open_actions()
+            .send_key(KeyCode::Enter) // Select first action - Edit Collection
+            .assert_empty();
+        // Event should be converted into a message appropriately
+        assert_matches!(harness.pop_message_now(), Message::CollectionEdit);
+    }
+
     /// Test "Copy URL" action, which is available via the Recipe List or Recipe
     /// panes
     #[rstest]
     fn test_copy_url(mut harness: TestHarness, terminal: TestTerminal) {
-        let expected_config = RequestConfig {
-            recipe_id: harness.collection.first_recipe_id().clone(),
-            profile_id: Some(harness.collection.first_profile_id().clone()),
-            options: BuildOptions::default(),
-        };
         let mut component = create_component(&mut harness, &terminal);
+
         component
-            .update_draw(Event::new_local(RecipeMenuAction::CopyUrl))
+            .int()
+            .send_key(KeyCode::Char('l')) // Select recipe list
+            .open_actions()
+            // Copy URL
+            .send_keys([KeyCode::Down, KeyCode::Enter])
             .assert_empty();
 
-        let request_config = assert_matches!(
-            harness.pop_message_now(),
-            Message::CopyRequestUrl(request_config) => request_config,
-        );
-        assert_eq!(request_config, expected_config);
-    }
-
-    /// Test "Copy Body" action, which is available via the Recipe List or
-    /// Recipe panes
-    #[rstest]
-    fn test_copy_body(mut harness: TestHarness, terminal: TestTerminal) {
-        let expected_config = RequestConfig {
-            recipe_id: harness.collection.first_recipe_id().clone(),
-            profile_id: Some(harness.collection.first_profile_id().clone()),
-            options: BuildOptions::default(),
-        };
-        let mut component = create_component(&mut harness, &terminal);
-        component
-            .update_draw(Event::new_local(RecipeMenuAction::CopyBody))
-            .assert_empty();
-
-        let request_config = assert_matches!(
-            harness.pop_message_now(),
-            Message::CopyRequestBody(request_config) => request_config,
-        );
-        assert_eq!(request_config, expected_config);
+        assert_matches!(harness.pop_message_now(), Message::CopyRequestUrl);
     }
 
     /// Test "Copy as cURL" action, which is available via the Recipe List or
     /// Recipe panes
     #[rstest]
     fn test_copy_as_curl(mut harness: TestHarness, terminal: TestTerminal) {
-        let expected_config = RequestConfig {
-            recipe_id: harness.collection.first_recipe_id().clone(),
-            profile_id: Some(harness.collection.first_profile_id().clone()),
-            options: BuildOptions::default(),
-        };
         let mut component = create_component(&mut harness, &terminal);
+
         component
-            .update_draw(Event::new_local(RecipeMenuAction::CopyCurl))
+            .int()
+            .send_key(KeyCode::Char('l')) // Select recipe list
+            .open_actions()
+            // Copy as cURL
+            .send_keys([KeyCode::Down, KeyCode::Down, KeyCode::Enter])
             .assert_empty();
 
-        let request_config = assert_matches!(
-            harness.pop_message_now(),
-            Message::CopyRequestCurl(request_config) => request_config,
-        );
-        assert_eq!(request_config, expected_config);
+        assert_matches!(harness.pop_message_now(), Message::CopyRequestCurl);
     }
 }
