@@ -2,7 +2,12 @@ use crate::{
     context::TuiContext,
     util::ResultReported,
     view::{
-        common::{table::Table, text_box::TextBox},
+        common::{
+            actions::{IntoMenuAction, MenuAction},
+            modal::Modal,
+            table::Table,
+            text_box::TextBox,
+        },
         component::{
             misc::TextBoxModal,
             recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
@@ -10,33 +15,35 @@ use crate::{
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
-        event::{Child, Event, EventHandler, Update},
+        event::{Child, Emitter, Event, EventHandler, OptionEvent},
         state::fixed_select::FixedSelectState,
         ViewContext,
     },
 };
 use derive_more::derive::Display;
 use ratatui::{
-    layout::Layout,
-    prelude::Constraint,
-    text::{Line, Span},
-    widgets::TableState,
-    Frame,
+    layout::Layout, prelude::Constraint, text::Span, widgets::TableState, Frame,
 };
 use slumber_config::Action;
 use slumber_core::{
     collection::{Authentication, RecipeId},
     template::Template,
 };
-use strum::{EnumCount, EnumIter};
+use strum::{EnumCount, EnumIter, IntoEnumIterator};
 
 /// Display authentication settings for a recipe
 #[derive(Debug)]
-pub struct AuthenticationDisplay(State);
+pub struct AuthenticationDisplay {
+    /// Emitter for the callback from editing the body
+    override_emitter: Emitter<SaveAuthenticationOverride>,
+    /// Emitter for menu actions
+    actions_emitter: Emitter<AuthenticationMenuAction>,
+    state: State,
+}
 
 impl AuthenticationDisplay {
     pub fn new(recipe_id: RecipeId, authentication: Authentication) -> Self {
-        let inner = match authentication {
+        let state = match authentication {
             Authentication::Basic { username, password } => {
                 let username = RecipeTemplate::new(
                     RecipeOverrideKey::auth_basic_username(recipe_id.clone()),
@@ -63,14 +70,18 @@ impl AuthenticationDisplay {
                 ),
             },
         };
-        Self(inner)
+        Self {
+            override_emitter: Default::default(),
+            actions_emitter: Default::default(),
+            state,
+        }
     }
 
     /// If the user has applied a temporary edit to the auth settings, get the
     /// override value. Return `None` to use the recipe's stock auth.
     pub fn override_value(&self) -> Option<Authentication> {
-        if self.0.is_overridden() {
-            Some(match &self.0 {
+        if self.state.is_overridden() {
+            Some(match &self.state {
                 State::Basic {
                     username, password, ..
                 } => Authentication::Basic {
@@ -89,22 +100,38 @@ impl AuthenticationDisplay {
 }
 
 impl EventHandler for AuthenticationDisplay {
-    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Update {
-        let action = event.action();
-        if let Some(Action::Edit) = action {
-            self.0.open_edit_modal();
-        } else if let Some(Action::Reset) = action {
-            self.0.reset_override();
-        } else if let Some(SaveAuthenticationOverride(value)) = event.local() {
-            self.0.set_override(value);
-        } else {
-            return Update::Propagate(event);
-        }
-        Update::Consumed
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event
+            .opt()
+            .action(|action, propagate| match action {
+                Action::Edit => {
+                    self.state.open_edit_modal(self.override_emitter)
+                }
+                Action::Reset => self.state.reset_override(),
+                _ => propagate.set(),
+            })
+            .emitted(
+                self.override_emitter,
+                |SaveAuthenticationOverride(value)| {
+                    self.state.set_override(&value)
+                },
+            )
+            .emitted(self.actions_emitter, |menu_action| match menu_action {
+                AuthenticationMenuAction::Edit => {
+                    self.state.open_edit_modal(self.override_emitter)
+                }
+                AuthenticationMenuAction::Reset => self.state.reset_override(),
+            })
+    }
+
+    fn menu_actions(&self) -> Vec<MenuAction> {
+        AuthenticationMenuAction::iter()
+            .map(MenuAction::with_data(self, self.actions_emitter))
+            .collect()
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
-        match &mut self.0 {
+        match &mut self.state {
             State::Basic { selected_field, .. } => {
                 vec![selected_field.to_child_mut()]
             }
@@ -116,11 +143,23 @@ impl EventHandler for AuthenticationDisplay {
 impl Draw for AuthenticationDisplay {
     fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
         let styles = &TuiContext::get().styles;
-
         let [label_area, content_area] =
             Layout::vertical([Constraint::Length(1), Constraint::Min(0)])
                 .areas(metadata.area());
-        let label = match &self.0 {
+
+        let label = match &self.state {
+            State::Basic { .. } => "Basic",
+            State::Bearer { .. } => "Bearer",
+        };
+        frame.render_widget(
+            Span::styled(
+                format!("Authentication Type: {label}"),
+                styles.text.title,
+            ),
+            label_area,
+        );
+
+        match &self.state {
             State::Basic {
                 username,
                 password,
@@ -140,23 +179,41 @@ impl Draw for AuthenticationDisplay {
                     content_area,
                     true,
                 );
-                "Basic"
             }
             State::Bearer { token } => {
                 frame.render_widget(token.preview().generate(), content_area);
-                "Bearer"
             }
-        };
-
-        let mut title: Line = Span::styled(
-            format!("Authentication Type: {label}"),
-            styles.text.title,
-        )
-        .into();
-        if self.0.is_overridden() {
-            title.push_span(Span::styled(" (edited)", styles.text.hint));
         }
-        frame.render_widget(title, label_area);
+    }
+}
+
+/// Local event to save a user's override value(s). Triggered from the edit
+/// modal. These will be raw string values, consumer has to parse them to
+/// templates.
+#[derive(Debug)]
+struct SaveAuthenticationOverride(String);
+
+#[derive(Copy, Clone, Debug, derive_more::Display, EnumIter)]
+enum AuthenticationMenuAction {
+    #[display("Edit Authentication")]
+    Edit,
+    #[display("Reset Authentication")]
+    Reset,
+}
+
+impl IntoMenuAction<AuthenticationDisplay> for AuthenticationMenuAction {
+    fn enabled(&self, data: &AuthenticationDisplay) -> bool {
+        match self {
+            Self::Edit => true,
+            Self::Reset => data.state.is_overridden(),
+        }
+    }
+
+    fn shortcut(&self, _: &AuthenticationDisplay) -> Option<Action> {
+        match self {
+            Self::Edit => Some(Action::Edit),
+            Self::Reset => Some(Action::Reset),
+        }
     }
 }
 
@@ -190,7 +247,7 @@ impl State {
     }
 
     /// Open a modal to let the user edit temporary override values
-    fn open_edit_modal(&self) {
+    fn open_edit_modal(&self, emitter: Emitter<SaveAuthenticationOverride>) {
         let (label, value) = match &self {
             Self::Basic {
                 username,
@@ -209,18 +266,17 @@ impl State {
                 ("bearer token", token.template().display())
             }
         };
-        ViewContext::open_modal(TextBoxModal::new(
+        TextBoxModal::new(
             format!("Edit {label}"),
             TextBox::default()
                 .default_value(value.into_owned())
                 .validator(|value| value.parse::<Template>().is_ok()),
-            |value| {
+            move |value| {
                 // Defer the state update into an event, so it can get &mut
-                ViewContext::push_event(Event::new_local(
-                    SaveAuthenticationOverride(value),
-                ))
+                emitter.emit(SaveAuthenticationOverride(value))
             },
-        ))
+        )
+        .open()
     }
 
     /// Override the value template for whichever field is selected, and
@@ -286,12 +342,6 @@ enum BasicFields {
     Password,
 }
 
-/// Local event to save a user's override value(s). Triggered from the edit
-/// modal. These will be raw string values, consumer has to parse them to
-/// templates.
-#[derive(Debug)]
-struct SaveAuthenticationOverride(String);
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -302,7 +352,7 @@ mod tests {
                 recipe_pane::persistence::RecipeOverrideValue,
                 RecipeOverrideStore,
             },
-            test_util::{TestComponent, WithModalQueue},
+            test_util::TestComponent,
         },
     };
     use crossterm::event::KeyCode;
@@ -319,22 +369,21 @@ mod tests {
         let mut component = TestComponent::new(
             &harness,
             &terminal,
-            WithModalQueue::new(AuthenticationDisplay::new(
-                RecipeId::factory(()),
-                authentication,
-            )),
-            (),
+            AuthenticationDisplay::new(RecipeId::factory(()), authentication),
         );
 
         // Check initial state
-        assert_eq!(component.data().inner().override_value(), None);
+        assert_eq!(component.data().override_value(), None);
 
         // Edit username
-        component.send_key(KeyCode::Char('e')).assert_empty();
-        component.send_text("!!!").assert_empty();
-        component.send_key(KeyCode::Enter).assert_empty();
+        component
+            .int()
+            .send_key(KeyCode::Char('e'))
+            .send_text("!!!")
+            .send_key(KeyCode::Enter)
+            .assert_empty();
         assert_eq!(
-            component.data().inner().override_value(),
+            component.data().override_value(),
             Some(Authentication::Basic {
                 username: "user1!!!".into(),
                 password: Some("hunter2".into())
@@ -342,16 +391,18 @@ mod tests {
         );
 
         // Reset username
-        component.send_key(KeyCode::Char('z')).assert_empty();
-        assert_eq!(component.data().inner().override_value(), None);
+        component.int().send_key(KeyCode::Char('z')).assert_empty();
+        assert_eq!(component.data().override_value(), None);
 
         // Edit password
-        component.send_key(KeyCode::Down).assert_empty();
-        component.send_key(KeyCode::Char('e')).assert_empty();
-        component.send_text("???").assert_empty();
-        component.send_key(KeyCode::Enter).assert_empty();
+        component
+            .int()
+            .send_keys([KeyCode::Down, KeyCode::Char('e')])
+            .send_text("???")
+            .send_key(KeyCode::Enter)
+            .assert_empty();
         assert_eq!(
-            component.data().inner().override_value(),
+            component.data().override_value(),
             Some(Authentication::Basic {
                 username: "user1".into(),
                 password: Some("hunter2???".into())
@@ -359,8 +410,8 @@ mod tests {
         );
 
         // Reset password
-        component.send_key(KeyCode::Char('z')).assert_empty();
-        assert_eq!(component.data().inner().override_value(), None);
+        component.int().send_key(KeyCode::Char('z')).assert_empty();
+        assert_eq!(component.data().override_value(), None);
     }
 
     #[rstest]
@@ -375,19 +426,16 @@ mod tests {
         let mut component = TestComponent::new(
             &harness,
             &terminal,
-            WithModalQueue::new(AuthenticationDisplay::new(
-                RecipeId::factory(()),
-                authentication,
-            )),
-            (),
+            AuthenticationDisplay::new(RecipeId::factory(()), authentication),
         );
 
         // Edit password
-        component.send_key(KeyCode::Down).assert_empty();
-        component.send_key(KeyCode::Char('e')).assert_empty();
-        component.send_key(KeyCode::Enter).assert_empty();
+        component
+            .int()
+            .send_keys([KeyCode::Down, KeyCode::Char('e'), KeyCode::Enter])
+            .assert_empty();
         assert_eq!(
-            component.data().inner().override_value(),
+            component.data().override_value(),
             Some(Authentication::Basic {
                 username: "user1".into(),
                 // None gets replaced by empty string. They're functionally
@@ -403,28 +451,48 @@ mod tests {
         let mut component = TestComponent::new(
             &harness,
             &terminal,
-            WithModalQueue::new(AuthenticationDisplay::new(
-                RecipeId::factory(()),
-                authentication,
-            )),
-            (),
+            AuthenticationDisplay::new(RecipeId::factory(()), authentication),
         );
 
         // Check initial state
-        assert_eq!(component.data().inner().override_value(), None);
+        assert_eq!(component.data().override_value(), None);
 
         // Edit token
-        component.send_key(KeyCode::Char('e')).assert_empty();
-        component.send_text("!!!").assert_empty();
-        component.send_key(KeyCode::Enter).assert_empty();
+        component
+            .int()
+            .send_key(KeyCode::Char('e'))
+            .send_text("!!!")
+            .send_key(KeyCode::Enter)
+            .assert_empty();
         assert_eq!(
-            component.data().inner().override_value(),
+            component.data().override_value(),
             Some(Authentication::Bearer("i am a token!!!".into()))
         );
 
         // Reset token
-        component.send_key(KeyCode::Char('z')).assert_empty();
-        assert_eq!(component.data().inner().override_value(), None);
+        component.int().send_key(KeyCode::Char('z')).assert_empty();
+        assert_eq!(component.data().override_value(), None);
+    }
+
+    /// Test edit menu action
+    #[rstest]
+    fn test_edit_action(harness: TestHarness, terminal: TestTerminal) {
+        let authentication = Authentication::Bearer("i am a token".into());
+        let mut component = TestComponent::new(
+            &harness,
+            &terminal,
+            AuthenticationDisplay::new(RecipeId::factory(()), authentication),
+        );
+
+        component
+            .int()
+            .open_actions()
+            .send_keys([KeyCode::Enter, KeyCode::Char('!'), KeyCode::Enter])
+            .assert_empty();
+        assert_eq!(
+            component.data().override_value(),
+            Some(Authentication::Bearer("i am a token!".into()))
+        );
     }
 
     /// Basic auth fields should load persisted overrides
@@ -447,7 +515,6 @@ mod tests {
             &harness,
             &terminal,
             AuthenticationDisplay::new(recipe_id, authentication),
-            (),
         );
 
         assert_eq!(
@@ -475,7 +542,6 @@ mod tests {
             &harness,
             &terminal,
             AuthenticationDisplay::new(recipe_id, authentication),
-            (),
         );
 
         assert_eq!(
