@@ -1,8 +1,9 @@
 use crate::{
-    context::TuiContext,
     util::ResultReported,
     view::{
         common::{
+            actions::{IntoMenuAction, MenuAction},
+            modal::Modal,
             table::{Table, ToggleRow},
             text_box::TextBox,
         },
@@ -13,8 +14,8 @@ use crate::{
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
-        event::{Child, Event, EventHandler, Update},
-        state::select::SelectState,
+        event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
+        state::select::{SelectState, SelectStateEvent, SelectStateEventType},
         util::persistence::{Persisted, PersistedKey, PersistedLazy},
         ViewContext,
     },
@@ -22,7 +23,6 @@ use crate::{
 use itertools::Itertools;
 use ratatui::{
     layout::Constraint,
-    text::Span,
     widgets::{Row, TableState},
     Frame,
 };
@@ -47,6 +47,12 @@ where
     RowSelectKey: PersistedKey<Value = Option<String>>,
     RowToggleKey: PersistedKey<Value = bool>,
 {
+    /// What kind of data we we storing? e.g. "Header"
+    noun: &'static str,
+    /// Emitter for the callback from editing a row
+    override_emitter: Emitter<SaveRecipeTableOverride>,
+    /// Emitter for menu actions
+    actions_emitter: Emitter<RecipeTableMenuAction>,
     select: Component<
         PersistedLazy<
             RowSelectKey,
@@ -61,6 +67,7 @@ where
     RowToggleKey: 'static + PersistedKey<Value = bool>,
 {
     pub fn new(
+        noun: &'static str,
         select_key: RowSelectKey,
         rows: impl IntoIterator<
             Item = (String, Template, RecipeOverrideKey, RowToggleKey),
@@ -81,9 +88,12 @@ where
             })
             .collect();
         let select = SelectState::builder(items)
-            .on_toggle(RowState::toggle)
+            .subscribe([SelectStateEventType::Toggle])
             .build();
         Self {
+            noun,
+            override_emitter: Default::default(),
+            actions_emitter: Default::default(),
             select: PersistedLazy::new(select_key, select).into(),
         }
     }
@@ -98,41 +108,73 @@ where
             })
             .collect()
     }
+
+    fn edit_selected_row(&self) {
+        if let Some(selected_row) = self.select.data().selected() {
+            selected_row.open_edit_modal(self.override_emitter);
+        }
+    }
+
+    fn reset_selected_row(&mut self) {
+        if let Some(selected_row) =
+            self.select.data_mut().get_mut().selected_mut()
+        {
+            selected_row.value.reset_override();
+        }
+    }
 }
 
 impl<RowSelectKey, RowToggleKey> EventHandler
     for RecipeFieldTable<RowSelectKey, RowToggleKey>
 where
     RowSelectKey: PersistedKey<Value = Option<String>>,
-    RowToggleKey: PersistedKey<Value = bool>,
+    RowToggleKey: 'static + PersistedKey<Value = bool>,
 {
-    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Update {
-        let action = event.action();
-        if let Some(Action::Edit) = action {
-            if let Some(selected_row) = self.select.data().selected() {
-                selected_row.open_edit_modal();
-            }
-            // Consume the event even if we have no rows, for consistency
-        } else if let Some(Action::Reset) = action {
-            if let Some(selected_row) =
-                self.select.data_mut().get_mut().selected_mut()
-            {
-                selected_row.value.reset_override();
-            }
-        } else if let Some(SaveRecipeTableOverride { row_index, value }) =
-            event.local()
-        {
-            // The row we're modifying *should* still be the selected row,
-            // because it shouldn't be possible to change the selection while
-            // the edit modal is open. It's safer to re-grab the modal by index
-            // though, just to be sure we've got the right one.
-            self.select.data_mut().get_mut().items_mut()[*row_index]
-                .value
-                .set_override(value);
-        } else {
-            return Update::Propagate(event);
-        }
-        Update::Consumed
+    fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
+        event
+            .opt()
+            .action(|action, propagate| match action {
+                // Consume the event even if we have no rows, for consistency
+                Action::Edit => self.edit_selected_row(),
+                Action::Reset => self.reset_selected_row(),
+                _ => propagate.set(),
+            })
+            .emitted(self.select.to_emitter(), |event| {
+                if let SelectStateEvent::Toggle(index) = event {
+                    self.select.data_mut().get_mut()[index].toggle();
+                }
+            })
+            .emitted(
+                self.override_emitter,
+                |SaveRecipeTableOverride { row_index, value }| {
+                    // The row we're modifying *should* still be the selected
+                    // row, because it shouldn't be possible to change the
+                    // selection while the edit modal is open. It's safer to
+                    // re-grab the modal by index though, just to be sure we've
+                    // got the right one.
+                    self.select.data_mut().get_mut().items_mut()[row_index]
+                        .value
+                        .set_override(&value);
+                },
+            )
+            .emitted(self.actions_emitter, |menu_action| match menu_action {
+                // The selected row can't change while the action menu is open,
+                // so we don't need to plumb the index/key through
+                RecipeTableMenuAction::Edit { .. } => self.edit_selected_row(),
+                RecipeTableMenuAction::Reset { .. } => {
+                    self.reset_selected_row()
+                }
+            })
+    }
+
+    fn menu_actions(&self) -> Vec<MenuAction> {
+        [
+            RecipeTableMenuAction::Edit { noun: self.noun },
+            RecipeTableMenuAction::Reset { noun: self.noun },
+        ]
+        .into_iter()
+        .map(MenuAction::with_data(self, self.actions_emitter))
+        .collect()
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -172,6 +214,53 @@ where
     }
 }
 
+/// Local event to modify a row's override template. Triggered from the edit
+/// modal
+#[derive(Debug)]
+struct SaveRecipeTableOverride {
+    row_index: usize,
+    value: String,
+}
+
+#[derive(Debug, derive_more::Display)]
+enum RecipeTableMenuAction {
+    #[display("Edit {noun}")]
+    Edit { noun: &'static str },
+    #[display("Reset {noun}")]
+    Reset { noun: &'static str },
+}
+
+impl<RowSelectKey, RowToggleKey>
+    IntoMenuAction<RecipeFieldTable<RowSelectKey, RowToggleKey>>
+    for RecipeTableMenuAction
+where
+    RowSelectKey: PersistedKey<Value = Option<String>>,
+    RowToggleKey: PersistedKey<Value = bool>,
+{
+    fn enabled(
+        &self,
+        data: &RecipeFieldTable<RowSelectKey, RowToggleKey>,
+    ) -> bool {
+        let selected = data.select.data().selected();
+        match self {
+            Self::Edit { .. } => selected.is_some(),
+            Self::Reset { .. } => {
+                selected.is_some_and(|row| row.value.is_overridden())
+            }
+        }
+    }
+
+    fn shortcut(
+        &self,
+        _: &RecipeFieldTable<RowSelectKey, RowToggleKey>,
+    ) -> Option<Action> {
+        match self {
+            Self::Edit { .. } => Some(Action::Edit),
+            Self::Reset { .. } => Some(Action::Reset),
+        }
+    }
+}
+
 #[derive(Clone)]
 pub struct RecipeFieldTableProps<'a> {
     /// Label for the left column in the table
@@ -213,13 +302,11 @@ impl<K: PersistedKey<Value = bool>> Generate for &RowState<K> {
     where
         Self: 'this,
     {
-        let styles = &TuiContext::get().styles;
-        let mut preview_text = self.value.preview().generate();
-        if self.value.is_overridden() {
-            preview_text.push_span(Span::styled(" (edited)", styles.text.hint));
-        }
-        ToggleRow::new([self.key.as_str().into(), preview_text], *self.enabled)
-            .generate()
+        ToggleRow::new(
+            [self.key.as_str().into(), self.value.preview().generate()],
+            *self.enabled,
+        )
+        .generate()
     }
 }
 
@@ -229,9 +316,9 @@ impl<K: PersistedKey<Value = bool>> RowState<K> {
     }
 
     /// Open a modal to create or edit the value's temporary override
-    fn open_edit_modal(&self) {
+    fn open_edit_modal(&self, emitter: Emitter<SaveRecipeTableOverride>) {
         let index = self.index;
-        ViewContext::open_modal(TextBoxModal::new(
+        TextBoxModal::new(
             format!("Edit value for {}", self.key),
             TextBox::default()
                 // Edit as a raw template
@@ -239,14 +326,13 @@ impl<K: PersistedKey<Value = bool>> RowState<K> {
                 .validator(|value| value.parse::<Template>().is_ok()),
             move |value| {
                 // Defer the state update into an event, so it can get &mut
-                ViewContext::push_event(Event::new_local(
-                    SaveRecipeTableOverride {
-                        row_index: index,
-                        value,
-                    },
-                ))
+                emitter.emit(SaveRecipeTableOverride {
+                    row_index: index,
+                    value,
+                });
             },
-        ));
+        )
+        .open();
     }
 
     /// Override the value template and re-render the preview
@@ -296,14 +382,6 @@ where
     }
 }
 
-/// Local event to modify a row's override template. Triggered from the edit
-/// modal
-#[derive(Debug)]
-struct SaveRecipeTableOverride {
-    row_index: usize,
-    value: String,
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -314,7 +392,7 @@ mod tests {
                 recipe_pane::persistence::RecipeOverrideValue,
                 RecipeOverrideStore,
             },
-            test_util::{TestComponent, WithModalQueue},
+            test_util::TestComponent,
         },
     };
     use crossterm::event::KeyCode;
@@ -358,15 +436,16 @@ mod tests {
                 },
             ),
         ];
-        let mut component = TestComponent::new(
+        let mut component = TestComponent::builder(
             &harness,
             &terminal,
-            RecipeFieldTable::new(TestRowKey(recipe_id.clone()), rows),
-            RecipeFieldTableProps {
-                key_header: "Key",
-                value_header: "Value",
-            },
-        );
+            RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
+        )
+        .with_props(RecipeFieldTableProps {
+            key_header: "Key",
+            value_header: "Value",
+        })
+        .build();
 
         // Check initial state
         assert_eq!(
@@ -375,8 +454,10 @@ mod tests {
         );
 
         // Disable the second row
-        component.send_key(KeyCode::Down).assert_empty();
-        component.send_key(KeyCode::Char(' ')).assert_empty();
+        component
+            .int()
+            .send_keys([KeyCode::Down, KeyCode::Char(' ')])
+            .assert_empty();
         let selected_row = component.data().select.data().selected().unwrap();
         assert_eq!(&selected_row.key, "row1");
         assert!(!*selected_row.enabled);
@@ -386,7 +467,7 @@ mod tests {
         );
 
         // Re-enable the row
-        component.send_key(KeyCode::Char(' ')).assert_empty();
+        component.int().send_key(KeyCode::Char(' ')).assert_empty();
         let selected_row = component.data().select.data().selected().unwrap();
         assert!(*selected_row.enabled);
         assert_eq!(
@@ -419,51 +500,83 @@ mod tests {
                 },
             ),
         ];
-        let mut component = TestComponent::new(
+        let mut component = TestComponent::builder(
             &harness,
             &terminal,
-            // We'll need a modal queue to handle the edit box
-            WithModalQueue::new(RecipeFieldTable::new(
-                TestRowKey(recipe_id.clone()),
-                rows,
-            )),
-            RecipeFieldTableProps {
-                key_header: "Key",
-                value_header: "Value",
-            },
-        );
+            RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
+        )
+        .with_props(RecipeFieldTableProps {
+            key_header: "Key",
+            value_header: "Value",
+        })
+        .build();
 
         // Check initial state
         assert_eq!(
-            component.data().inner().to_build_overrides(),
+            component.data().to_build_overrides(),
             BuildFieldOverrides::default()
         );
 
         // Edit the second row
-        component.send_key(KeyCode::Down).assert_empty();
-        component.send_key(KeyCode::Char('e')).assert_empty(); // Open the modal
-        component.send_text("!!!").assert_empty();
-        component.send_key(KeyCode::Enter).assert_empty();
+        component
+            .int()
+            // Open the modal
+            .send_keys([KeyCode::Down, KeyCode::Char('e')])
+            .send_text("!!!")
+            .send_key(KeyCode::Enter)
+            .assert_empty();
 
-        let selected_row =
-            component.data().inner().select.data().selected().unwrap();
+        let selected_row = component.data().select.data().selected().unwrap();
         assert_eq!(&selected_row.key, "row1");
         assert!(selected_row.value.is_overridden());
         assert_eq!(selected_row.value.template().display(), "value1!!!");
         assert_eq!(
-            component.data().inner().to_build_overrides(),
+            component.data().to_build_overrides(),
             [(1, BuildFieldOverride::Override("value1!!!".into()))]
                 .into_iter()
                 .collect(),
         );
 
         // Reset edited state
-        component.send_key(KeyCode::Char('z')).assert_empty();
-        let selected_row =
-            component.data().inner().select.data().selected().unwrap();
+        component.int().send_key(KeyCode::Char('z')).assert_empty();
+        let selected_row = component.data().select.data().selected().unwrap();
         assert!(!selected_row.value.is_overridden());
     }
 
+    /// Test Edit menu action
+    #[rstest]
+    fn test_edit_action(harness: TestHarness, terminal: TestTerminal) {
+        let recipe_id = RecipeId::factory(());
+        let rows = [(
+            "row0".into(),
+            "value0".into(),
+            RecipeOverrideKey::query_param(recipe_id.clone(), 0),
+            TestRowToggleKey {
+                recipe_id: recipe_id.clone(),
+                key: "row0".into(),
+            },
+        )];
+        let mut component = TestComponent::builder(
+            &harness,
+            &terminal,
+            RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
+        )
+        .with_props(RecipeFieldTableProps {
+            key_header: "Key",
+            value_header: "Value",
+        })
+        .build();
+
+        component
+            .int()
+            .open_actions()
+            .send_keys([KeyCode::Enter, KeyCode::Char('!'), KeyCode::Enter])
+            .assert_empty();
+
+        let selected_row = component.data().select.data().selected().unwrap();
+        assert_eq!(selected_row.value.template().display(), "value0!");
+    }
+
     /// Override templates should be loaded from the store on init
     #[rstest]
     fn test_persisted_override(harness: TestHarness, terminal: TestTerminal) {
@@ -496,22 +609,19 @@ mod tests {
                 },
             ),
         ];
-        let component = TestComponent::new(
+        let component = TestComponent::builder(
             &harness,
             &terminal,
-            // We'll need a modal queue to handle the edit box
-            WithModalQueue::new(RecipeFieldTable::new(
-                TestRowKey(recipe_id.clone()),
-                rows,
-            )),
-            RecipeFieldTableProps {
-                key_header: "Key",
-                value_header: "Value",
-            },
-        );
+            RecipeFieldTable::new("Row", TestRowKey(recipe_id.clone()), rows),
+        )
+        .with_props(RecipeFieldTableProps {
+            key_header: "Key",
+            value_header: "Value",
+        })
+        .build();
 
         assert_eq!(
-            component.data().inner().to_build_overrides(),
+            component.data().to_build_overrides(),
             [
                 (0, BuildFieldOverride::Override("p0".into())),
                 (1, BuildFieldOverride::Override("p1".into()))
